\name{AsciiToInt}
\alias{AsciiToInt}
\alias{ichar}
\alias{chars8bit}
\alias{strcodes}
\title{Character to and from Integer Codes Conversion}
\description{
  \code{AsciiToInt} returns \code{\link{integer}} codes in \code{0:255}
  for each (one byte) character in \code{strings}.  \code{ichar} is an
  alias for it, for old S compatibility.

  \code{strcodes} implements in \R the basic engine for translating
  characters to corresponding integer codes.

  \code{chars8bit()} is the \emph{inverse} function of
  \code{AsciiToint}, producing (one byte) characters from integer codes.
}
\usage{
AsciiToInt(strings)
     ichar(strings)
chars8bit(i = 0:255)
strcodes(x, table = chars8bit(0:255))
}
\arguments{
  \item{strings, x}{\code{\link{character}} vector.}
  \item{i}{numeric (integer) vector of values in \code{0:255}.}
  \item{table}{a vector of (unique) character strings, typically of one
  character each.}
}
\details{
  Only codes in \code{0:127} make up the ASCII encoding which should be
  identical for all \R versions, whereas the \emph{\sQuote{upper}} half
  is often determined from the ISO-8859-1 (aka \dQuote{ISO-Latin 1)}
  encoding, but may well differ, depending on the locale setting, see
  also \code{\link{Sys.setlocale}}.
}
\value{
  \code{AsciiToInt} (and hence \code{ichar}) and \code{chars8bit} return a
  vector of the same length as their argument.

  \code{strcodes(x, tab)} returns a \code{\link{list}} of the same
  \code{\link{length}} and \code{\link{names}} as \code{x} with list
  components of integer vectors with codes in \code{0:255}.
}
\author{Martin Maechler, partly in 1991 for S-plus}
\examples{
chars8bit(65:70)#-> "A" "B" .. "F"
stopifnot(identical(LETTERS,   chars8bit(65:90)),
          identical(AsciiToInt(LETTERS), 65:90))

strcodes(c(a= "ABC", ch="1234", place = "Zrich"))
## gives
\dontrun{% Rout
$a
[1] 65 66 67

$ch
[1] 49 50 51 52

$place
[1]  90 252 114 105  99 104
}
}
\keyword{manip}
%\keyword{utilities}

\eof
% This is also sym.linked into
% Martin's WpDensity package /u/maechler/R/Pkgs/WpDensity/
\name{D1D2}
\alias{D1D2}
\title{Numerical Derivatives of (x,y) Data via Smoothing Splines}
\description{
  Compute numerical derivatives of \eqn{f()} given observations
  \code{(x,y)}, using cubic smoothing splines with GCV, see
  \code{\link[stats]{smooth.spline}}.  In other words, estimate \eqn{f'()}
  and/or \eqn{f''()} for the model
  \deqn{Y_i = f(x_i) + E_i, \ \ i = 1,\dots n,}
}
\usage{
D1D2(x, y, xout = x, spar.offset = 0.1384, deriv = 1:2, spl.spar = NULL)
}
\arguments{
  \item{x,y}{numeric vectors of same length, supposedly from a model
    \code{y ~ f(x)}.}
  \item{xout}{abscissa values at which to evaluate the derivatives.}
  \item{spar.offset}{numeric fudge added to the smoothing parameter,
    see \code{spl.par} below.}
  \item{deriv}{integer in \code{1:2} indicating which
    derivatives are to be computed.}
  \item{spl.spar}{direct smoothing parameter for \code{smooth.spline}.
    If it is \code{NULL} (as per default), the smoothing parameter used
    will be \code{spar.offset + sp$spar}, where \code{sp$spar} is the GCV
  estimated smoothing parameter, see \code{\link{smooth.spline}}.}
}
\details{
  It is well known that for derivative estimation, the optimal smoothing
  parameter is larger (more smoothing) than for the function itself.
  \code{spar.offset} is really just a \emph{fudge} offset added to the
  smoothing parameter. Note that in \R's implementation of
  \code{\link{smooth.spline}}, \code{spar} is really on the
  \eqn{\log\lambda} scale.

  When \code{deriv = 1:2} (as per default), both derivatives are
  estimated with the \emph{same} smoothing parameter which is suboptimal
  for the single functions individually.  Another possibility is to call
  \code{D1D2(*, deriv = k)} twice with \code{k = 1} and \code{k = 2} and
  use a \emph{larger} smoothing parameter for the second derivative.
}
\value{
  a list with several components,
  \item{x}{the abscissae values at which the derivative(s) are evaluated.}
  \item{D1}{if \code{deriv} contains 1, estimated values of
    \eqn{f'(x_i)} where \eqn{x_i} are the values from \code{xout}.}
  \item{D2}{if \code{deriv} contains 2, estimated values of \eqn{f''(x_i)}.}
  \item{spar}{the \bold{s}moothing \bold{par}ameter used in the (final)
    \code{smooth.spline} call.}
  \item{df}{the equivalent \bold{d}egrees of \bold{f}reedom in that
    \code{smooth.spline} call.}
}
\author{Martin Maechler, in 1992 (for S).}
\seealso{\code{\link{D2ss}} which calls \code{smooth.spline} twice,
  first on \code{y}, then on the \eqn{f'(x_i)} values;
  \code{\link[stats]{smooth.spline}} on which it relies completely.
}
\examples{
 set.seed(8840)
 x <- runif(100, 0,10)
 y <- sin(x) + rnorm(100)/4

 op <- par(mfrow = c(2,1))
 plot(x,y)
 lines(ss <- smooth.spline(x,y), col = 4)
 str(ss[c("df", "spar")])
 if(is.R()) plot(cos, 0, 10, ylim = c(-1.5,1.5), lwd=2) else { # Splus
   xx <- seq(0,10, len=201); plot(xx, cos(xx), type = 'l', ylim = c(-1.5,1.5))
 }
 title(expression("Estimating f'() :  " * frac(d,dx) * sin(x) == cos(x)))
 offs <- c(-0.1, 0, 0.1, 0.2, 0.3)
 i <- 1
 for(off in offs) {
   d12 <- D1D2(x,y, spar.offset = off)
   lines(d12$x, d12$D1, col = i <- i+1)
 }
 legend(2,1.6, c("true cos()",paste("sp.off. = ", format(offs))), lwd=1,
        col = 1:(1+length(offs)), cex = 0.8, bg = NA)
 par(op)
}
\keyword{smooth}

\eof
\name{D2ss}
\alias{D2ss}
\alias{D1ss}
\alias{D1tr}
\title{Numerical Derivatives of (x,y) Data  (via Smoothing Splines)}
\description{
  Compute the numerical first or 2nd derivatives of \eqn{f()} given
  observations \code{(x[i], y ~= f(x[i]))}.

  \code{D1tr} is the \emph{\bold{tr}ivial} discrete first derivative
  using simple difference ratios, whereas \code{D1ss} and \code{D2ss}
  use cubic smoothing splines (see \code{\link[stats]{smooth.spline}})
  to estimate first or second derivatives, respectively.

  \code{D2ss} first uses \code{smooth.spline} for the first derivative
  \eqn{f'()} and then applies the same to the predicted values
  \eqn{\hat f'(t_i)}{f'^(t[i])} (where \eqn{t_i}{t[i]} are the values of
  \code{xout}) to find   \eqn{\hat f''(t_i)}{f''^(t[i])}.
}
\usage{
D1tr(y, x = 1)

D1ss(x, y, xout = x, spar.offset = 0.1384, spl.spar=NULL)
D2ss(x, y, xout = x, spar.offset = 0.1384, spl.spar=NULL)
}
\arguments{
  \item{x,y}{numeric vectors of same length, supposedly from a model
    \code{y ~ f(x)}.  For \code{D1tr()}, \code{x} can have length one
    and then gets the meaning of \eqn{h = \Delta x}.}
  \item{xout}{abscissa values at which to evaluate the derivatives.}
  \item{spar.offset}{numeric fudge added to the smoothing parameter(s),
    see \code{spl.par} below.  Note that the current default is there
    for historical reasons only, and we often  would recommend to use
    \code{spar.offset = 0} instead.}
  \item{spl.spar}{direct smoothing parameter(s) for \code{smooth.spline}.
    If it is \code{NULL} (as per default), the smoothing parameter used
    will be \code{spar.offset + sp$spar}, where \code{sp$spar} is the GCV
  estimated smoothing parameter for \emph{both} smooths, see
  \code{\link{smooth.spline}}.}
}
\details{
  It is well known that for derivative estimation, the optimal smoothing
  parameter is larger (more smoothing needed) than for the function itself.
  \code{spar.offset} is really just a \emph{fudge} offset added to the
  smoothing parameters. Note that in \R's implementation of
  \code{\link{smooth.spline}}, \code{spar} is really on the
  \eqn{\log\lambda} scale.
%
%   When \code{deriv = 1:2} (as per default), both derivatives are
%   estimated with the \emph{same} smoothing parameter which is suboptimal
%   for the single functions individually.  Another possibility is to call
%   \code{D1D2(*, deriv = k)} twice with \code{k = 1} and \code{k = 2} and
%   use a \emph{larger} smoothing parameter for the second derivative.
}
\value{
  \code{D1tr()} and \code{D1ss()} return a numeric vector of the length
  of \code{y} or \code{xout}, respectively.

  \code{D2ss()} returns a list with components
  \item{x}{the abscissae values (= \code{xout}) at which the
    derivative(s) are evaluated.}
  \item{y}{estimated values of \eqn{f''(x_i)}.}
  \item{spl.spar}{numeric vector of length 2, contain the \code{spar}
    arguments to the two \code{smooth.spline} calls.}
  \item{spar.offset}{as specified on input (maybe rep()eated to length 2).}
}
\author{Martin Maechler, in 1992 (for S).}
\seealso{\code{\link{D1D2}} which directly uses the 2nd derivative of
  the smoothing spline;  \code{\link{smooth.spline}}.
}
\examples{

## First Derivative  --- spar.off = 0  ok "asymptotically" (?)
set.seed(330)
mult.fig(12)
for(i in 1:12) {
  x <- runif(500, 0,10); y <- sin(x) + rnorm(500)/4
  f1 <- D1ss(x=x,y=y, spar.off=0.0)
  plot(x,f1, ylim = range(c(-1,1,f1)))
  curve(cos(x), col=3, add= TRUE)
}

 set.seed(8840)
 x <- runif(100, 0,10)
 y <- sin(x) + rnorm(100)/4

 op <- par(mfrow = c(2,1))
 plot(x,y)
 lines(ss <- smooth.spline(x,y), col = 4)
 str(ss[c("df", "spar")])
 xx <- seq(0,10, len=201)
 plot(xx, -sin(xx), type = 'l', ylim = c(-1.5,1.5))
 title(expression("Estimating f''() :  " * frac(d^2,dx^2) * sin(x) == -sin(x)))
 offs <- c(0.05, 0.1, 0.1348, 0.2)
 i <- 1
 for(off in offs) {
   d12 <- D2ss(x,y, spar.offset = off)
   lines(d12, col = i <- i+1)
 }
 legend(2,1.6, c("true :  -sin(x)",paste("sp.off. = ", format(offs))), lwd=1,
        col = 1:(1+length(offs)), cex = 0.8, bg = NA)
 par(op)
}
\keyword{smooth}

\eof
\name{Deprecated}% <<< would like "special name": don't check \arguments{}
\title{Deprecated 'sfsmisc' Functions}
% NOTE: ../R/Deprecated.R   must be synchronized with this!
%       ~~~~~~~~~~~~~~~~~
\alias{nna}
\alias{digits}
\alias{digits.v}
\alias{subtit}
\alias{p.triangle}
\alias{p.panelL}
\alias{p.panelS}
\alias{p.pllines}
\alias{tapply.num}
\alias{list2mat}
%------ PLEASE: put \alias{.} here for EACH !
\description{
  These functions are provided for compatibility with older versions of
  the \pkg{sfsmisc} package only, and may be defunct as soon as of the
  next release.
}
\usage{
nna(data)

digits.v(nvec, base = 2, num.bits = 1 + floor(log(max(nvec),base)))
digits(n, base = 10)

subtit(t)

p.panelL(x,y)
p.panelS(x,y,df = 4)
p.pllines(x,y,group,lty=c(1,3,2,4),\dots)

p.triangle(mat, label= "*", text.ecken = rep("",3), dreieck = TRUE)

list2mat(x, check = TRUE)
}
\arguments{
% nna
  \item{data}{vector, matrix or data frame.}
% digits, digits.v
  \item{nvec, n}{non-negative integer (vector).}
  \item{base}{positive integer}
  \item{num.bits}{positive integer}
% subtit():
  \item{t}{character string.}
% p.triangle():
\item{mat}{matrix mit \% A in der 1. Kolonne, \% B in der 2.. (range 0:1)}
\item{label}{text of length \code{mat[,1]} for labeling points.}
\item{text.ecken}{\code{character(3)} text for the three corners.}
\item{dreieck}{logical; if true, draw the triangle border.}
% p.panelL() & p.panelS():
\item{x,y}{panel function (or plot or general) arguments:
  numeric vectors of same length.}
\item{df}{specify (equivalent) degrees of freedom for smoothing spline
  in \code{p.panelS}.}
% list2mat() - `x' already documented
\item{check}{logical specifiying if it should be checked that list
  components have same length.}
% p.pllines(x,y,group,lty=c(1,3,2,4),\dots)
\item{group}{grouping vector of same length as x or y.}
\item{lty}{line types to use for the groups (recycled).}
\item{\dots}{further arguments passed to (plot) methods.}
}
\details{
  \code{nna} was a simpler version of the generic \code{\link{na.omit}}
  function; the latter should be used instead.

  \code{digits.v} and \code{digits} are old vector and \dQuote{scalar}
  versions of \code{\link{digitsBase}}.

  \code{subtit(t)} is really just a convenience abbreviation for
  \code{mtext(t, side = 3, line = 0)} which you should rather use.

  \code{p.panelL} and \code{p.panelS} are simple two-line functions to
  be used as \code{panel} arguments in \code{\link{pairs}} or
  \code{\link{coplot}}.  However, for such simple panels, it's rather
  recommended to use \emph{anonymous} functions, i.e.,
  \code{panel = function(x,y) { ....}} in the call to
  \code{\link{coplot}()} (e.g.).

  \code{p.pllines} is deprecated because basic \R graphics (but not
  S-PLUS) provide its   functionality directly: Use \code{plot(x,y, lty
    = group, type = 'l', ...)}.

  \code{p.triangle} is deprecated because package \pkg{vcd} has a much
  more versatile \code{\link[vcd]{ternaryplot}} function.

  \code{tapply.num} has simply be renamed to \code{\link{tapplySimpl}}
  which you should use instead.

  \code{list2mat(x)} is usually the same as \code{sapply(x, c)} (where
  the latter does not construct column names where \code{x} has no names).
}
\keyword{documentation}


\eof
\name{KSd}
\alias{KSd}
\title{Approximate Critical Values for Kolmogorov-Smirnov's D}
\description{
  Computes the critical value for Kolmogorov-Smirnov's \eqn{D_n}, for
  sample sizes \eqn{n \ge 10}{n >= 10} and confidence level 95\%.
}
\details{
  Based on tables values given in the reference below.
  For \eqn{n\le 80}{n <= 80} uses interpolations from exact values, elsewhere
  uses asymptotic approximation.
}
\usage{
KSd(n)
}
\arguments{
  \item{n}{the sample size, \code{n >= 10}.}
}
\value{
  The critical value for D (two-sided) for significance level 0.05 (or
  confidence level 95\%).
}
\references{
  Peter J. Bickel and Kjell A. Doksum (1977),
  \emph{Mathematical Statistics: Basic Ideas and Selected Topics}.
  Holden Day.
  Section 9.6 and table IX.
}
\author{Kjetil Halvorsen and Martin Maechler}

\seealso{Is used from \code{\link{ecdf.ksCI}}.}

\examples{
KSd(90)
KSd(1:9)# now works

op <- par(mfrow=c(2,1))
  plot(KSd, 10, 150)# nice
  abline(v = c(75,85), col = "gray")
  plot(KSd, 79, 81, n = 1001)# *very* tiny discontinuity at 80
par(op)
}
\keyword{distribution}

\eof
\name{QUnif}
\alias{QUnif}
\alias{sHalton}
%
\concept{Quasi Monte Carlo}
\concept{low discrepancy sequence}
\concept{space filling}
%
\title{Quasi Randum Numbers via Halton Sequences}
\description{
  These functions provide quasi random numbers or \emph{space filling} or
  \emph{low discrepancy} sequences in the \eqn{p}-dimensional unit cube.
}
\usage{
sHalton(n.max, n.min = 1, base = 2, leap = 1)
 QUnif (n, min = 0, max = 1, n.min = 1, p, leap = 1)
}
\arguments{
  \item{n.max}{maximal (sequence) number.}
  \item{n.min}{minimal sequence number.}
  \item{n}{number of \eqn{p}-dimensional points generated in
    \code{QUnif}.  By default, \code{n.min = 1, leap = 1} and
    the maximal sequence number is \code{n.max = n.min + (n-1)*leap}.}
  \item{base}{integer \eqn{\ge 2}{>= 2}: The base with respect to which
    the Halton sequence is built.}
  \item{min, max}{lower and upper limits of the univariate intervals.
    Must be of length 1 or \code{p}.}
  \item{p}{dimensionality of space (the unit cube) in which points are
    generated.}
  \item{leap}{integer indicating (if \eqn{> 1}) if the series should be
    leaped, i.e., only every \code{leap}th entry should be taken.}
}
\value{
  \code{sHalton(n,m)} returns a numeric vector of length \code{n-m+1} of
  values in \eqn{[0,1]}.

  \code{QUnif(n, min, max, n.min, p=p)} generates \code{n-n.min+1}
  p-dimensional points in \eqn{[min,max]^p} returning a numeric matrix
  with p columns.
}
\note{
  For \code{leap} Kocis and Whiten recommend values of
  \eqn{L=31,61,149,409}, and particularly the \eqn{L=409} for dimensions
  up to 400.
}
\references{
  James Gentle (1998)
  \emph{Random Number Generation and Monte Carlo Simulation}; sec.\ 6.3.
  Springer.

  Kocis, L. and Whiten, W.J. (1997)
  Computationsl Investigations of Low-Discrepancy Sequences.
  \emph{ACM Transactions of Mathematical Software} \bold{23}, 2, 266--294.
}
\author{Martin Maechler}
\examples{
32*sHalton(20, base=2)

stopifnot(sHalton(20, base=3, leap=2) ==
          sHalton(20, base=3)[1+2*(0:9)])
## ------- a 2D Visualization -------

Uplot <- function(xy, axes=FALSE, xlab="", ylab="", ...) {
  plot(xy, xaxs="i", yaxs="i", xlim=0:1, ylim=0:1, xpd = FALSE,
       axes=axes, xlab=xlab, ylab=ylab, ...)
  box(lty=2, col="gray40")
}

do4 <- function(n, ...) {
  op <- mult.fig(4, main=paste("n =", n,": Quasi vs. (Pseudo) Random"),
                 marP=c(-2,-2,-1,0))$old.par
  on.exit(par(op))
  for(i in 1:2) {
     Uplot(QUnif(n, p=2), main="QUnif", ...)
     Uplot(cbind(runif(n), runif(n)), main="runif", ...)
  }
}
do4(100)
do4(500)
do4(1000, cex = 0.8, col="slateblue")
do4(10000, pch= ".", col="slateblue")
do4(40000, pch= ".", col="slateblue")
}
\keyword{math}
\keyword{multivariate}
\keyword{datagen}

\eof
\name{TA.plot}
\alias{TA.plot}
\title{Tukey-Anscombe Plot (residual vs. fitted) of a Linear Model. }
\description{
  From a linear (or glm) model fitted, produce the so-called Tukey-Anscombe
  plot.  Useful (optional) additions include:   0-line, lowess smooth,
  2sigma lines,  and automatic labeling of observations.
}
\usage{
TA.plot(lm.res,
        fit= fitted(lm.res), res= residuals(lm.res, type="pearson"),
        labels= NULL, main= mk.main(), xlab = "Fitted values",
        draw.smooth= n >= 10, show.call = TRUE, show.2sigma= TRUE,
        lo.iter = NULL, lo.cex= NULL,
        par0line  = list(lty = 2, col = 2),
        parSmooth = list(lwd = 1.5, lty = 4, col = 3),
        parSigma  = list(lwd = 1.2, lty = 3, col = 4),
        \dots)
}
\arguments{
  \item{lm.res}{Result of \code{\link{lm}(..)}, \code{\link{aov}(..)},
    \code{\link{glm}(..)} or a similar object.}
  \item{fit}{fitted values; you probably want the default here.}
  \item{res}{residuals to use. Default: \bold{Weighted} ("Pearson") residuals if
    weights have been used for the model fit.}
  \item{labels}{strings to use as plotting symbols for each point.
    Default(NULL): extract observations' names or use its sequence number.
    Use, e.g., "*" to get simple \code{*} symbols.
  }
  \item{main}{main title to plot. Default: sophisticated, resulting in
    something like "Tukey-Anscombe Plot of :   y \~ x"  constructed from
    \code{call(.)} in \code{lm.res}.
  }
  \item{xlab}{x-axis lable for plot.}
  \item{draw.smooth}{logical; if \code{TRUE}, draw a \code{lowess} smoother
    (with automatic smoothing fraction).}
  \item{show.call}{logical; if \code{TRUE}, write the "call"ing syntax with
    which the fit was done.}
  \item{show.2sigma}{logical; if \code{TRUE}, draw horizontal lines at
    \eqn{\pm 2\sigma}{+- 2 sigma} where \eqn{\sigma} is \code{mad(resid)}.}
  \item{lo.iter}{positive integer, giving the number of lowess
    robustness iterations. The default depends on the model and
    is \code{0} for non Gaussian \code{\link{glm}}'s.}
  \item{lo.cex}{character expansion ("cex") for lowess and other
    marginal texts.}
  \item{par0line}{a list of arguments (with reasonable defaults) to be passed to
    \code{\link{abline}(.)} when drawing the x-axis, i.e.,
    the \eqn{y = 0} line.}
  \item{parSmooth, parSigma}{each a list of arguments (with reasonable
    default) for drawing the smooth curve (if \code{draw.smooth} is
    true), or the horizontal sigma boundaries (if \code{show.2sigma} is
    true) respectively.}
  \item{\dots}{further graphical parameters are passed to
    \code{\link{n.plot}(.)}.}
}
\section{Side Effects}{
  The above mentioned plot is produced on the current graphic device.
}
\author{
  Martin Maechler, Seminar fuer Statistik, ETH Zurich, Switzerland;
  \email{maechler@stat.math.ethz.ch}
}
\seealso{\code{\link{plot.lm}} which also does a QQ normal plot and more.
}
\examples{
data(stackloss)
TA.plot(lm(stack.loss ~ stack.x))

example(airquality)
summary(lmO <- lm(Ozone ~ ., data= airquality))
TA.plot(lmO)
TA.plot(lmO, label = "O") # instead of case numbers

if(FALSE) { %%  from S-plus
 TA.plot(lm(cost ~ age+type+car.age, claims, weights=number, na.action=na.omit))
}

##--- for  aov(.) : -------------
data(Gun, package = "nlme")
TA.plot( aov(rounds ~ Method + Physique/Team, data = Gun))

##--- Not so clear what it means for GLM, but: ------
if(require(rpart)) { # for the two datasets only
 data(solder, package = "rpart")
 TA.plot(glm(skips ~ ., data = solder, family = poisson), cex= .6)

 data(kyphosis, package = "rpart")
 TA.plot(glm(Kyphosis ~ poly(Age,2) + Start, data=kyphosis, family = binomial),
	 cex=.75) # smaller title and plotting characters
}
}
\keyword{models}
\keyword{regression}


\eof
\name{bl.string}
\alias{bl.string}
\title{Blank String}
\description{
  Simple constructor of a ``blank'' string of given string length.
}
\usage{
bl.string(no)
}
\arguments{
  \item{no}{non-negative integer.}
}
\value{
  A ``blank'' string (i.e. \code{\link{character}(1)}) fulfilling
  \code{n == nchar(bl.string(n))}.
}
\author{Martin Maechler, early 1990's.}
\seealso{\code{\link{paste}}, \code{\link{character}}, \code{\link{nchar}}.}
\examples{
r <- sapply(0:8, function(n) ccat(bl.string(n),n))
cbind(r)

bl.string # shows the very simple function definition
}
\keyword{character}

\eof
\name{boxplot.matrix}
\alias{boxplot.matrix}
\title{Draw a Boxplot for each Column (Row) of a Matrix}
\description{
  Interpreting the columns (or rows) of a matrix as different groups, draw a
  boxplot for each.
}
\usage{
boxplot.matrix(x, use.cols = TRUE, \dots)
}
\arguments{
  \item{x}{a numeric matrix.}
  \item{use.cols}{logical indicating if columns (by default) or rows
    (\code{use.cols=FALSE}) should be plotted.}
  \item{\dots}{Further arguments to \code{\link{boxplot}}.}
}
\note{
  This function is really superfluous nowadays, since
  \code{boxplot(data.frame(x), ...)} is sufficient.
}
\value{
  A list as for \code{\link{boxplot}}.
}
\author{Martin Maechler, 1995}
\seealso{\code{\link{boxplot.default}} which already works with
  data.frames; \code{\link{boxplot.formula}}, \code{\link{plot.factor}} which
  work with (the more general concept) of a grouping factor.
}
\examples{
## Very similar to the example in ?boxplot
mat <- cbind(Uni05 = (1:100)/21, Norm = rnorm(100),
             T5 = rt(100, df = 5), Gam2 = rgamma(100, shape = 2))
boxplot.matrix(mat, main = "boxplot.matrix(...., main = ...)",
               notch = TRUE, col = 1:4)
}
\keyword{hplot}


\eof
\name{col01scale}
\alias{col01scale}
\alias{colcenter}
\title{Matrix Scaling Utilities}
\description{
  \code{col01scale} and \code{colcenter} (re)scale the columns of a
  matrix.  These are simple one-line utilities, mainly with a didactical
  purpose.
}
\usage{
colcenter (mat)
col01scale(mat, scale.func = function(x) diff(range(x)), location.func = mean)
}
\arguments{
  \item{mat}{numeric matrix, to rescaled.}
  \item{scale.func, location.func}{two functions mapping a numeric
    vector to a single number.}
}
\value{
  a matrix with the same attributes as the input \code{mat}.
}
\author{Martin Maechler}
\seealso{The standard \R function \code{\link{scale}()}.}
\examples{
## See the simple function definitions:

colcenter ## simply one line

col01scale# almost as simple
}
\keyword{array}

\eof
\name{compresid2way}
\alias{compresid2way}
\title{Plot Components + Residuals for Two Factors}
\description{
  For an analysis of variance or regression with (at least) two factors:
  Plot components + residuals for two factors according to Tukey's
  \dQuote{forget-it plot}. Try it!
}
\usage{
compresid2way(aov, data=NULL, fac=1:2, label = TRUE, numlabel = FALSE,
             xlab=NULL, ylab=NULL, main=NULL,
             col=c(2,3,4,4), lty=c(1,1,2,4), pch=c(1,2))
}
\arguments{
  \item{aov}{either an \code{\link{aov}} object with a formula of the form
%       \code{y \~{} a + b}, where  \code{a}  and  \code{b}  are factors,
        \code{y ~ a + b}, where  \code{a}  and  \code{b}  are factors,
        or such a formula.}
  \item{data}{data frame containing  \code{a}  and  \code{b}.}
  \item{fac}{the two factors used for plotting.  Either column numbers or
    names for argument \code{data}.}
  \item{label}{logical indicating if levels of factors should be shown
    in the plot.}
  \item{numlabel}{logical indicating if effects of factors will be shown
    in the plot.}
  \item{xlab,ylab,main}{the usual \code{\link{title}} components, here
    with a non-trivial default constructed from \code{aov} and the
    component factors used.}
  \item{col,lty,pch}{colors, line types, plotting characters to be used
    for plotting [1] positive residuals, [2] negative residuals,
    [3] grid, [4] labels.  If \code{pch} is sufficiently long, it will be used
    as the list of individual symbols for plotting the y values.}
}
\details{For a two-way analysis of variance, the plot shows the additive
  components of the fits for the two factors by the intersections of a
  grid, along with the residuals.
  The observed values of the target variable are identical to the
  vertical coordinate.

  The application of the function has been extended to cover more
  complicated models. The components of the fit for two factors are
  shown as just described, and the residuals are added. The result is a
  \dQuote{component plus residual} plot for two factors in one display.
}
\value{Invisibly, a list with components
  \item{compy}{data.frame containing the component effects of the two
    factors, and combined effects plus residual}
  \item{coef}{coefficients: Intercept and effects of the factors}
}
\references{
  F. Mosteller and J. W. Tukey (1977)
  \emph{Data Analysis and Regression: A Second Course in Statistics}.
  {Addison-Wesley}, {Reading, Mass.}, p. 176.

  John W. Tukey (1977)
  \emph{Exploratory Data Analysis}.
  {Addison-Wesley, Reading, Mass.}, p. 381.
}
\author{Werner Stahel \email{stahel@stat.math.ethz.ch}}
\seealso{\code{\link{interaction.plot}}}
\examples{
 ## From Venables and Ripley (2002) p.165.
 N <- c(0,1,0,1,1,1,0,0,0,1,1,0,1,1,0,0,1,0,1,0,1,1,0,0)
 P <- c(1,1,0,0,0,1,0,1,1,1,0,0,0,1,0,1,1,0,0,1,0,1,1,0)
 K <- c(1,0,0,1,0,1,1,0,0,1,0,1,0,1,1,0,0,0,1,1,1,0,1,0)
 yield <- c(49.5,62.8,46.8,57.0,59.8,58.5,55.5,56.0,62.8,55.8,69.5,55.0,
            62.0,48.8,45.5,44.2,52.0,51.5,49.8,48.8,57.2,59.0,53.2,56.0)
 npk <- data.frame(block=gl(6,4), N=factor(N), P=factor(P),
                   K=factor(K), yield=yield)
 npk.cr <- compresid2way(yield ~ N+P+K, data=npk, fac=c("P","K"))

 ## Fisher's 1926 data on potatoe yield
 data(potatoes)
 pot.aov <- aov(yield ~ nitrogen+potash+pos, data=potatoes)
 compresid2way(pot.aov, pch=as.character(potatoes$pos))

 compresid2way(yield~nitrogen+potash, data=subset(potatoes, pos == 2))

 ## 2 x 3 design :
 data(warpbreaks)
 summary(fm1 <- aov(breaks ~ wool + tension, data = warpbreaks))
 compresid2way(fm1)
}
\keyword{hplot}

\eof
\name{cum.Vert.funkt}
\alias{cum.Vert.funkt}

\title{Kumulative Verteilung Aufzeichnen}
\description{
  Kumulative Verteilung von \code{x} aufzeichnen, auf Wunsch auch Median
  und Quartile.

  This is just an old German language version of
  \code{\link[stats]{plot.ecdf}()} used for teaching at ETHZ.
}
\usage{
cum.Vert.funkt(x, Quartile = TRUE, titel = TRUE, Datum = TRUE,
               rang.axis = n <= 20, xlab = "", main = "", \dots)
}
\arguments{
  \item{x}{numeric vector whose empirical distribution should be plotted.}
  \item{Quartile}{logical indicating if all 3 non-trivial quartiles
    should be drawn.}
  \item{titel}{logical indicating if a German title should be drawn.}
  \item{Datum}{logical indicating if \code{\link{p.datum}} should be added.}
  \item{rang.axis}{logical indicating if all the ranks should be marked at
    the y-axis.  Defaults to true if there are not more than 20 observations.}
  \item{xlab, main}{x-axis label and main title; default to empty.}
  \item{\dots}{}
}
\value{
  the return value of \code{\link{plotStep}()} which is called
  internally, \emph{invisibly}.
}
\author{Martin Maechler et al.}
\seealso{\code{\link{plotStep}} on which it is based; but you should
  really consider using \code{\link[stats]{plot.ecdf}()} from the
  \pkg{stats} package instead of this.}
\examples{
cum.Vert.funkt(runif(12))
cum.Vert.funkt(runif(20))

Z <- rnorm(50)
cum.Vert.funkt(Z)
}
\keyword{hplot}

\eof
\name{diagDA}
\title{Diagonal Discriminant Analysis}
\alias{diagDA}
\alias{dDA}
\alias{print.dDA}
\alias{predict.dDA}
\description{
  This function implements a simple Gaussian maximum likelihood
  discriminant rule, for diagonal class covariance matrices.
}
\usage{
dDA(x, cll, pool = TRUE)
\method{predict}{dDA}(object, newdata, pool = object$pool, \dots)
\method{print}{dDA}(x, \dots)

diagDA(ls, cll, ts, pool = TRUE)
}
\arguments{
  \item{x,ls}{learning set data matrix, with rows corresponding to
    cases (i.e., mRNA samples) and columns to predictor variables
    (i.e., genes).}
  \item{cll}{class labels for learning set, must be consecutive integers.}
  \item{object}{object of class \code{dDA}.}
  \item{ts, newdata}{test set (prediction) data matrix, with rows corresponding
    to cases and columns to predictor variables.}
  \item{pool}{logical flag.  If true (by default), the covariance matrices
    are assumed to be constant across classes and the discriminant rule
    is linear in the data.  Otherwise (\code{pool= FALSE}), the
    covariance matrices may vary across classes and the discriminant
    rule is quadratic in the data.}
  \item{\dots}{further arguments passed to and from methods.}
}
\value{
  \code{dDA()} returns an object of class \code{dDA} for which there are
  \code{\link{print}} and \code{\link{predict}} methods.  The latter
  returns the same as \code{diagDA()}:

  \code{diagDA()} returns an integer vector of class predictions for the
  test set.
}
\references{
  S. Dudoit, J. Fridlyand, and T. P. Speed. (2000)
  Comparison of Discrimination Methods for the Classification of Tumors
  Using Gene Expression Data.
  (Statistics, UC Berkeley, June 2000, Tech Report \#576)
}
\author{
  Sandrine Dudoit, \email{sandrine@stat.berkeley.edu}  and\cr
  Jane Fridlyand, \email{janef@stat.berkeley.edu} originally wrote
  \code{stat.diag.da()} in CRAN package \pkg{sma} which was modified
  for speedup by Martin Maechler \email{maechler@R-project.org}
  who also introduced \code{dDA} etc.
}
\seealso{\code{\link[MASS]{lda}} and \code{\link[MASS]{qda}} from the
  \pkg{MASS} package.
}
\examples{
## two artificial examples by Andreas Greutert:
d1 <- data.frame(x = c(1, 5, 5, 5, 10, 25, 25, 25, 25, 29),
                 y = c(4, 1, 2, 4,  4,  4,     6:8,     7))
n.plot(d1)
library(cluster)
(cl1P <- pam(d1,k=4)$cluster) # 4 surprising clusters
with(d1, points(x+0.5, y, col = cl1P, pch =cl1P))

i1 <- c(1,3,5,6)
tr1 <- d1[-i1,]
cl1. <- c(1,2,1,2,1,3)
cl1  <- c(2,2,1,1,1,3)
plot(tr1, cex=2, col = cl1, pch = 20+cl1)
(dd.<- diagDA(tr1, cl1., ts = d1[ i1,]))# ok
(dd <- diagDA(tr1, cl1 , ts = d1[ i1,]))# ok, too!
points(d1[ i1,], pch = 10, cex=3, col = dd)

## use new fit + predict instead :
(r1 <- dDA(tr1, cl1))
(r1.<- dDA(tr1, cl1.))
stopifnot(dd == predict(r1,  new = d1[ i1,]),
          dd.== predict(r1., new = d1[ i1,]))

plot(tr1, cex=2, col = cl1, bg = cl1, pch = 20+cl1,
     xlim=c(1,30), ylim= c(0,10))
xy <- cbind(x= runif(500, min=1,max=30), y = runif(500, min=0, max=10))
points(xy, cex= 0.5, col = predict(r1, new = xy))
abline(v=c( mean(c(5,25)), mean(c(25,29))))
}
\keyword{models}
\keyword{classif}


\eof
\name{diagX}
\alias{diagX}
\title{The ``Other'' Diagonal Matrix}
\description{
  Compute the \emph{other} diagonal identity matrix.
  The result is basically a \emph{fast} version of \code{diag(n)[, n:1]}.
}
\usage{
diagX(n)
}
\arguments{
  \item{n}{positive integer.}
}
\value{
  a numeric \eqn{n \times n}{n * n} matrix with many zeros -- apart from
  \code{1}s in the \emph{other} diagonal.
}
\author{Martin Maechler, 1992.}
\seealso{\code{\link{diag}}.}
\examples{
diagX(4)
for(m in 1:5)
  stopifnot(identical(diagX(m), diag(m)[, m:1, drop = FALSE]))
}
\keyword{array}
\keyword{utilities}

\eof
\name{digitsBase}
\alias{digitsBase}
\title{Digit/Bit Representation of Integers in any Base}
\description{
  Compute the vector of \dQuote{digits} \eqn{A} of the \code{base} \eqn{b}
  representation of a number \eqn{N}, \eqn{N = \sum_{k=0}^M  A_{M-k} b ^ k}{%
    N = sum(k = 0:M ; A[M-k] * b^k)}.
}
\usage{
digitsBase(x, base = 2, ndigits = 1 + floor(log(max(x), base)))
}
\arguments{
  \item{x}{non-negative integer (vector) whose base \code{base}
    digits are wanted.}
  \item{base}{integer, at least 2 specifying the base for representation.}
  \item{ndigits}{number of bits/digits to use.}
}
\value{
  a \code{\link{matrix}} \code{m} where \code{m[,i]}
  corresponds to \code{x[i]}.

  \code{c( result )} then contains the blocks in proper order ..
}
\note{\code{digits} and \code{digits.v} are now deprecated and will be
  removed from the \pkg{sfsmisc} package.
}
\author{Martin Maechler, Dec 4, 1991 (for S-plus; then called \code{digits.v}).}
\examples{
digitsBase(0:12, 8) #-- octal representation
empty.dimnames(digitsBase(0:33, 2)) # binary

## This may be handy for just one number (and default decimal):
digits <- function(n, base = 10) drop(digitsBase(n, base = base))
digits(128, base = 8) # 2 0 0

hexdig <- c(0:9, LETTERS[1:6]) #-- hexadesimal coding :
i <- c(1:20, 100:120) # e.g.
M <- digitsBase(i, 16)
cM <- hexdig[1 + M]; dim(cM) <- dim(M)
sub("^0+","", apply(cM,2,paste, collapse=""))
}
\keyword{arith}
\keyword{utilities}

\eof
\name{ecdf.ksCI}
\alias{ecdf.ksCI}
\title{
  Plot the Empirical Distribution Function Together With
  95\% Confidence Curves.
}
\description{
  Plots the empirical distribution function for univariate data,
  together with upper and lower simultaneous 95\% confidence curves.
}
\usage{
ecdf.ksCI(x, main = NULL, sub = NULL, xlab = deparse(substitute(x)),
          ci.col = "red", \dots)
}
\arguments{
  \item{x}{\code{x} numerical vector of observations.}
  \item{main,sub,xlab}{arguments passed to \code{\link{title}}.}
  \item{ci.col}{color for confidence interval lines.}
  \item{\dots}{\code{\dots} arguments given to \code{\link{plot.stepfun}}.}
}
\value{
  Nothing. Used for its side effect, to produce a plot.
}
\references{Bickel and Doksum, see \code{\link{KSd}}.
}
\author{Kjetil Halvorsen}

\note{Presently, will only work if \code{length(x)} > 9. }

\seealso{\code{\link{ecdf}} and \code{\link{plot.stepfun}} in package
          \code{\link{stepfun}}. }

\examples{
ecdf.ksCI( rchisq(50,3) )
}
\keyword{hplot}

\eof
\name{ellipsePoints}
\alias{ellipsePoints}
\title{Compute Radially Equispaced Points on Ellipse}
\description{
  Compute points on (the boundary of) an ellipse which is given by
  elementary geometric parameters.
}
\usage{
ellipsePoints(a, b, alpha = 0, loc = c(0, 0), n = 201)
}
\arguments{
  \item{a,b}{length of half axes in (x,y) direction.}
  \item{alpha}{angle (in degrees) giving the orientation of the ellipse,
    i.e., the original (x,y)-axis ellipse is rotated by \code{angle}.}
  \item{loc}{center (LOCation) of the ellipse.}
  \item{n}{number of points to generate.}
}
\value{
  A numeric matrix of dimension \code{n x 2}, each row containing the
  (x,y) coordinates of a point.
}
\author{Martin Maechler, March 2002.}
\seealso{the \file{ellipse} package and \code{\link[cluster]{ellipsoidhull}}
 and \code{\link[cluster]{ellipsoidPoints}}
  in the \file{cluster} package.
}
\examples{
## Simple Ellipse, centered at (0,0), x-/y- axis parallel:
ep <- ellipsePoints(2,5)
str(ep)
plot(ep, type="n",asp=1) ; polygon(ep, col = 2)

## rotate by 30 degrees :
plot(ellipsePoints(2,5, alpha = 30), asp=1)
abline(h=0,v=0,col="gray")
abline(a=0,b= tan( 30 *pi/180), col=2, lty = 2)
abline(a=0,b= tan(120 *pi/180), col=3, lty = 2)

if(dev.interactive()) {
  ## Movie : rotating ellipse  :
  nTurns <- 4 # #{full 360 deg turns}
  for(al in 1:(nTurns*360)) {
      ep <- ellipsePoints(3,6, alpha=al, loc = c(5,2))
      plot(ep,type="l",xlim=c(-1,11),ylim=c(-4,8),
	   asp=1, axes = FALSE, xlab="", ylab="")
  }

  ## Movie : rotating _filled_ ellipse {less nice to look at}
  for(al in 1:180) {
      ep <- ellipsePoints(3,6, alpha=al, loc = c(5,2))
      plot(ep,type="n",xlim=c(-1,11),ylim=c(-4,8),
	   asp=1, axes = FALSE, xlab="", ylab="")
      polygon(ep,col=2,border=3,lwd=2.5)
  }
}# only if interactive
}
\keyword{iplot}
\keyword{utilities}

\eof
\name{empty.dimnames}
\alias{empty.dimnames}
\title{Empty Dimnames of an Array.}
\description{
  `Remove' all dimension names from an array for compact printing.
}
\usage{
empty.dimnames(a)
}
\arguments{
  \item{a}{an \code{\link{array}}, especially a matrix.}
}
\value{
  Returns \code{a} with its dimnames replaced by empty character strings.
}
\author{Bill Venables / Martin Maechler, Sept 1993.}
\seealso{\code{\link{unname}} \emph{removes} the dimnames.}
\examples{
empty.dimnames(diag(5)) # looks much nicer

(a <- matrix(-9:10, 4,5))
empty.dimnames(a) # nicer, right?
}
\keyword{array}
\keyword{print}
\keyword{utilities}

\eof
\name{errbar}
\title{Scatter Plot with Error Bars}
\alias{errbar}
\description{
  Draws a scatter plot, adding vertical ``error bars'' to all the points.
}
\usage{
errbar(x, y, yplus, yminus, cap = 0.015,
       xlab= deparse(substitute(x)),
       ylab= deparse(substitute(y)), \dots)
}
\arguments{
  \item{x}{vector of x values.}
  \item{y}{vector of y values.}
  \item{yplus}{vector of y values: the tops of the error bars.}
  \item{yminus}{vector of y values: the bottoms of the error bars.}
  \item{cap}{the width of the little lines at the tops and bottoms of
    the error bars in units of the width of the plot.  Default is 0.015.}
  \item{xlab, ylab}{axis labels for the plot, as in
    \code{\link{plot.default}}.}
  \item{\dots}{Graphical parameters (see \code{\link{par}}) may also
    be supplied as arguments to this function.}
}
\author{Originally Charles Geyer, U.Chicago, early 1991;
  then Martin Mchler.
}
\examples{
y <- rnorm(10); d <- 1 + .1*rnorm(10)
errbar(1:10, y, y + d, y - d, main="Error Bars example")
}
\keyword{hplot}


\eof
\name{f.robftest}
\alias{f.robftest}
\title{Robust F-Test: Wald test for multiple coefficients of rlm() Object.}
\description{
  Compute a robust F-Test, i.e., a Wald test for multiple coefficients
  of an \code{\link[MASS]{rlm}} Object.
}
\usage{
f.robftest(object, var = -1)
}
\arguments{
  \item{object}{result of \code{\link{rlm}()}.}
  \item{var}{variables.  Either their names or their indices;  the
    default, \code{-1} means all \emph{but} the intercept.}
}
\details{
  This builds heavily on \code{\link[MASS]{summary.rlm}()}, the
  \code{\link{summary}} method for \code{\link{rlm}} results.
}
\value{
  An object of class \code{"htest"}, hence with the standard print
  methods for hypothesis tests.  This is basically a list with components
  \item{statistic}{the F statistic, according to ...}% FIXME
  \item{df}{numerator and denominator degrees of freedom.}
  \item{data.name}{(extracted from input \code{object}.)}
  \item{alternative}{\code{"two.sided"}, always.}
  \item{p.value}{the P-value, using an F-test on \code{statistic} and
    \code{df[1:2]}.}
}
\references{
  FIXME --- Need some here !
}
\author{Werner Stahel, Jul.2000; updates by Martin Maechler.}
\seealso{\code{\link{rlm}}, \code{\link{summary.aov}}, etc.}
\examples{
if(require("MASS")) {
  ## same data as  example(rlm)
  data(stackloss)
  summary(rsl <- rlm(stack.loss ~ ., stackloss))
  f.robftest(rsl)
 } else  " forget it "

}
\keyword{robust}
\keyword{htest}


\eof
\name{glob2rx}
\alias{glob2rx}
\title{Change Wildcard or Globbing Pattern into Regular Expression}
\description{
  Change \emph{wildcard} aka \emph{globbing} (or \dQuote{ls} like)
  pattern into the corresponding regular expression (\code{\link{regexp}}).
}
\usage{
glob2rx(pattern)
}
\arguments{
  \item{pattern}{character vector}
}
\value{
  a character vector of the same length as the input \code{pattern}
  where the \dQuote{wild card} is translated to the corresponding
  regular expression.
}
\author{Martin Maechler, Unix/sed based version, 1991; current: 2004}
\seealso{\code{\link{regexp}} about regular expression,
  \code{\link{sub}}, etc about substitutions using regexps.}
\examples{
stopifnot(glob2rx("abc.*") == "^abc\\\\.",% '\\' doubled in Rd file!
          glob2rx("a?b.*") == "^a.b\\\\.",
          glob2rx("*.doc") == "^.*\\\\.doc$",
          glob2rx("*.t*")  == "^.*\\\\.t",
          glob2rx("*.t??") == "^.*\\\\.t..$")
}
\keyword{file}
\keyword{character}
\keyword{utilities}


\eof
\name{hatMat}
\alias{hatMat}
\title{Hat Matrix of a Smoother}
\description{
  Compute the hat matrix or smoother matrix, of `any' (linear) smoother,
  smoothing splines, by default.
}
\usage{
hatMat(x, trace= FALSE,
       pred.sm = function(x, y, ...)
                 predict(smooth.spline(x, y, ...), x = x)$y,
       \dots)
}
\arguments{
  \item{x}{numeric vector or matrix.}
  \item{trace}{logical indicating if the whole hat matrix, or only its
    trace, i.e. the sum of the diagonal values should be computed.}
  \item{pred.sm}{a function of at least two arguments \code{(x,y)}
    which returns fitted values, i.e. \eqn{\hat{y}}{y.hat}, of length
    compatible to \code{x} (and \code{y}).}
  \item{\dots}{optionally further arguments to the smoother function
    \code{pred.sm}.}
}
\value{
  The hat matrix \eqn{H} (if \code{trace = FALSE} as per default) or
  a number, \eqn{tr(H)}, the \emph{trace} of \eqn{H}, i.e.,
  \eqn{\sum_i H_{ii}}{sum(i) H[i,i]}.

  Note that \code{dim(H) == c(n, n)} where \code{n <- length(x)} also in
  the case where some x values are duplicated (aka \emph{ties}).
}
\references{
  Hastie and Tibshirani (1990).
  \emph{Generalized Additive Models}.
  Chapman \& Hall.
}
\author{Martin Maechler \email{maechler@stat.math.ethz.ch}}
\seealso{\code{\link{smooth.spline}}, etc}
\examples{
require(stats) # for smooth.spline() or loess()

x1 <- c(1:4, 7:12)
H1 <- hatMat(x1, spar = 0.5)

matplot(x1, H1, type = "l", main = "columns of smoother hat matrix")

## Example `pred.sm' arguments for hatMat() :
pspl <- function(x,y,...) predict(smooth.spline(x,y, ...), x = x)$y
ploes <-function(x,y,...) predict(loess(y ~ x, ...))
pksm <- function(x,y,...) ksmooth(x,y,"normal", x.points=x, ...)$y
pRmean <- function(x,y,...) run.mean(y, ...)
pRline <- function(x,y,...) run.line(x,y, ...)$y

all.equal(sum(diag((hatMat(c(1:4, 7:12), df = 4)))),
                    hatMat(c(1:4, 7:12), df = 4, trace = TRUE), tol = 1e-12)
## TRUE

%% should also show the above functions

%% should test these also for x with ties
}
\keyword{smooth}
\keyword{regression}

\eof
\name{hist.bxp}
\alias{hist.bxp}
\title{Plot a Histogram and a Boxplot}
\description{
  Creates a histogram and a horizontal boxplot on the current graphics
  device.
}
\usage{
hist.bxp(x, nclass, breaks, probability=FALSE, include.lowest=TRUE,
         xlab = deparse(substitute(x)),
         \dots,
         width=0.2, boxcol=3, medcol=0, medlwd=5, whisklty=2, staplelty=1)
}
\arguments{
  \item{x}{numeric vector of data for histogram. Missing values
    (\code{NA}s) are allowed.}
  \item{nclass}{
    recommendation for the number of classes (i.e., bars) the histogram should
    have. The default is a number proportional to the logarithm of the length
    of \code{x}.
  }
  \item{breaks}{
    vector of the break points for the bars of the histogram. The count in the
    i-th bar is \code{sum(breaks[i] < x <= breaks[i+1])}
    except that if \code{include.lowest} is \code{TRUE} (the default),
    the first bar also includes points equal to \code{breaks[1]}.  If
    omitted, evenly-spaced break points are determined from
    \code{nclass} and the extremes of the data.
  }
  \item{probability}{
    logical flag: if \code{TRUE}, the histogram will be scaled as a probability
    density; the sum of the bar heights times bar widths will equal \code{1}. If
    \code{FALSE}, the heights of the bars will be counts.
  }
  \item{include.lowest}{
    If \code{TRUE} (the default), the lowest bar will include data
    points equal to the lowest break, otherwise it will act like the
    other bars (see the description of the \code{breaks} argument).
  }
  \item{xlab}{character or expression for x axis labeling.}
  \item{\dots}{additional arguments to \code{\link{barplot}}.  The
    \code{\link{hist}} function uses the function \code{barplot} to do
    the actual plotting; consequently, arguments to the \code{barplot}
    function that control shading, etc., can also be given to
    \code{hist}. See the \code{barplot} documentation for arguments
    \code{angle}, \code{density}, \code{col}, and  \code{inside}. Do not
    use the \code{space} or \code{histo} arguments.
  }
  \item{width}{
    width of the box relative to the height of the histogram. DEFAULT is
    \code{0.2}.}
  \item{boxcol}{filled box color. A color of \code{0} can be used to
    designate filling with the background color. The default is to fill
    with color \code{3}.}
  \item{medcol}{
    the color of the median line. The special value, \code{NA},
    indicates the current plotting color (\code{par("col")}). The
    default is \code{0} (the background color). If \code{boxcol=0} and
    \code{medcol} is not explicitly specified this is set to the current
    plotting color (\code{par("col")}).
  }
  \item{medlwd}{
    median line width.  The special value \code{NA}, is used to indicate
    the current line width (\code{par("lwd")}).  The default is \code{5}.
  }
  \item{whisklty}{
    whisker line type. The special value \code{NA} indicates the
    current line type (\code{par("lty")}). The default is \code{2}
    (dotted line).}
  \item{staplelty}{
    staple (whisker end cap) line type. The special value \code{NA}
    indicates the current line type (\code{par("lty")}). The default is
    \code{1} (solid line).

    Graphical parameters (see \code{\link{par}}) may also
    be supplied as arguments to this function.
    In addition, the high-level graphics arguments described under
    \code{par} and the arguments to \code{title} may be supplied to this
    function.}
}
\details{
  If \code{include.lowest} is \code{FALSE} the bottom breakpoint must be
  strictly less than the minimum of the data, otherwise (the default) it
  must be less than or equal to the minimum of the data. The top
  breakpoint must be greater than or equal to the maximum of the data.
}
\author{S-Plus: Markus Keller, Christian Keller; port to \R: Martin Mchler.
}
\seealso{\code{\link{hist}}, \code{\link{barplot}},
  \code{\link{boxplot}}, \code{\link{rug}} and
  \code{\link[Hmisc]{scat1d}} in the \pkg{Hmisc} package.
}
\examples{
 lab <- "50 samples from a t distribution with 5 d.f."
 mult.fig(2*3, main = "Hist() + Rug()   and    Hist.bxp(*)")
 for(i in 1:3) {
   my.sample <-  rt(50, 5)
   hist(my.sample, main=lab); rug(my.sample)# for 50 obs., this is ok, too..
   hist.bxp(my.sample, main=lab)
 }
}
\keyword{hplot}


\eof
\name{huberM}
\alias{huberM}
\title{Safe (generalized) Huber M-Estimator of Location}
\description{
  (Generalized) Huber M-estimator of location with MAD scale, being
  sensible also when the scale is zero where \code{\link[MASS]{huber}()}
  returns an error.
}
\usage{
huberM(x, k = 1.5, tol = 1e-06, mu = median(x), s = mad(x, center=mu),
       warn0scale = getOption("verbose"))
}
\arguments{
  \item{x}{numeric vector.}
  \item{k}{positive factor; the algorithm winsorizes at \code{k}
    standard deviations.}
  \item{tol}{convergence tolerance.}
  \item{mu}{initial location estimator.}
  \item{s}{scale estimator held constant through the iterations.}
  \item{warn0scale}{logical; if true, and \code{s} is 0 and
    \code{length(x) > 1}, this will be warned about.}
}
\value{
  list of location and scale parameters, and number of iterations used.
  \item{mu}{location estimate}
  \item{s}{the \code{s} argument,MAD scale estimate -- unless \code{s} is}
  \item{it}{the number of \dQuote{Huber iterations} used.}
}
\author{Martin Maechler building on the MASS code mentioned.}
\references{
  Huber, P. J. (1981)
  \emph{Robust Statistics.}
  Wiley.
}
\seealso{
  \code{\link[MASS]{hubers}} (and \code{huber}) in package \pkg{MASS};
  \code{\link{mad}}.
}
\examples{
huberM(c(1:9, 1000))
mad  (c(1:9, 1000))
mad  (rep(9, 100))
huberM(rep(9, 100))
}
\keyword{robust}

\eof
\name{integrate.xy}
\alias{integrate.xy}
\title{Cheap Numerical Integration through Data points.}
\description{
  Given \eqn{(x_i, f_i)} where \eqn{f_i = f(x_i)}, compute a cheap
  approximation of \eqn{\int_a^b f(x) dx}{integral(a .. b) f(x) dx}.
}
\usage{
integrate.xy(x, fx, a, b, use.spline=TRUE, xtol=2e-08)
}
\arguments{
  \item{x}{abscissa values.}
  \item{fx}{corresponding values of \eqn{f(x)}.}
  \item{a,b}{the boundaries of integration; these default to min(x) and
    max(x) respectively.}
  \item{use.spline}{logical; if TRUE use an interpolating spline.}
  \item{xtol}{tolerance factor, typically around
    \code{sqrt(.Machine$double.eps)} ......(fixme)....}
}
\details{
  Note that this is really not good for noisy \code{fx} values;
  probably a smoothing spline should be used in that case.

  Also, we are not yet using Romberg in order to improve the trapezoid
  rule.  This would be quite an improvement in equidistant cases.
}
\value{
  the approximate integral.
}
\author{Martin Maechler, May 1994 (for S).}
\seealso{\code{\link{integrate}} for numerical integration of
  \emph{functions}.}
\examples{
 x <- 1:4
 integrate.xy(x, exp(x))
 print(exp(4) - exp(1), digits = 10) # the true integral

 for(n in c(10, 20,50,100, 200)) {
   x <- seq(1,4, len = n)
   cat(formatC(n,wid=4), formatC(integrate.xy(x, exp(x)), dig = 9),"\n")
 }
}
\keyword{math}
\keyword{utilities}

\eof
\name{inv.seq}
\alias{inv.seq}
\title{Inverse seq() -- Short Expression for Index Vector}
\description{
  Compute a short expression for a given integer vector, typically
  an index, that can be expressed shortly, using \code{\link{:}} etc.
}
\usage{
inv.seq(i)
}
\arguments{
  \item{i}{vector of (usually increasing) integers.}
}
%%\details{
%%
%%}
\value{
  an \code{\link{expression}} which can be \code{\link{eval}()}ed to
  return the original \code{i}.
}
\author{Martin Maechler, Oct. 1995}
\seealso{\code{\link{rle}} for another kind of integer vector coding.}
\examples{
(rr <- inv.seq(i1 <- c(3:12, 20:24, 27, 30:33)))
eval(rr)
stopifnot(eval(rr) == i1)

e2 <- expression(c(20:13, 3:12, -1:-4, 27, 30:31))
(r2 <- inv.seq(i2 <- eval(e2)))
stopifnot(identical(e2,r2))
}
\keyword{arith}
\keyword{utilities}

\eof
\name{iterate.lin.recursion}
\alias{iterate.lin.recursion}
\title{Generate Sequence Iterating a Linear Recursion}
\description{
  Generate numeric sequences applying a linear recursion \code{nr.it} times.
}
\usage{
iterate.lin.recursion(x, coeff, delta = 0, nr.it)
}
\arguments{
  \item{x}{numeric vector with \emph{initial values}, i.e., specifying
    the beginning of the resulting sequence; must be of length (larger
    or) equal to \code{length(coeff)}.}
  \item{coeff}{coefficient vector of the linear recursion.}
  \item{delta}{numeric scalar added to each term; defaults to 0.  If not
    zero, determines the linear drift component.}
  \item{nr.it}{integer, number of iterations.}
}
\value{
  numeric vector, say \code{r}, of length \code{n + nr.it}, where
  \code{n = length(x)}. Initialized as \code{r[1:n] = x}, the recursion
  is \code{r[k+1] = sum(coeff * r[(k-m+1):k])}, where \code{m = length(coeff)}.
}
\note{
  Depending on the zeroes of the characteristic polynomial of \code{coeff},
  there are three cases, of convergence, oszillation and divergence.
}
\author{Martin Maechler}
\seealso{\code{\link{seq}} can be regarded as a trivial special case.}
\examples{
## The Fibonacci sequence:
iterate.lin.recursion(0:1, c(1,1), nr = 12)
## 0 1 1 2 3 5 8 13 21 34 55 89 144 233

## seq() as a special case:
stopifnot(iterate.lin.recursion(4,1, d=2, nr=20)
          == seq(4, by=2, length=1+20))

## ``Deterministic AR(2)'' :
round(iterate.lin.recursion(1:4, c(-0.7, 0.9), d = 2, nr=15), dig=3)
## slowly decaying :
plot(ts(iterate.lin.recursion(1:4, c(-0.9, 0.95), nr=150)))
}
\keyword{arith}

\eof
\name{last}
\alias{last}
\title{Get Last Elements of a Vector}
\description{
  Extract the last elements of a vector.
}
\usage{
last(x, length.out = 1, na.rm = FALSE)
}
\arguments{
  \item{x}{any vector.}
  \item{length.out}{integer indicating how many element are desired.  If
    positive, return the \code{length.out} last elements of \code{x}; if
    negative, the last \code{length.out} elements are \emph{dropped}.
  }
  \item{na.rm}{logical indicating if the last non-missing value (if any)
    shall be returned.  By default (it is \code{FALSE} and) the last
    elements (whatever its values) are returned.}
}
\value{
  a vector of length \code{abs(length.out)} of \emph{last} values from \code{x}.
}
\author{Werner Stahel (\email{stahel@stat.math.ethz.ch}), and independently,
        Philippe Grosjean (\email{phgrosjean@sciviews.org}),
        Frdric Ibanez (\email{ibanez@obs-vlfr.fr}).}
\note{
  This function may eventually be deprecated for the new (R 1.9.0)
  function \code{\link{tail}()}.

  Useful for the \code{\link[pastecs]{turnogram}()} function in package
  \pkg{pastecs}.
}

\seealso{\code{\link[pastecs]{first}}, \code{\link[pastecs]{turnogram}}
}
\examples{
a <- c(NA, 1, 2, NA, 3, 4, NA)
last(a)
last(a, na.rm=TRUE)

last(a, length = 2)
last(a, length = -3)
}
\keyword{ manip }

\eof
\name{linesHyperb.lm}
\alias{linesHyperb.lm}
\title{Plot Confidence or Prediction Hyperbolas around a Regression Line}
\description{
  Add confidence/prediction hyperbolas for \eqn{y(x_0)}
  to a plot with data or regression line.
}
\usage{
linesHyperb.lm(object, c.prob=0.95, confidence=FALSE,
            k=if (confidence) Inf else 1,
            col=2, lty=2, do.abline=TRUE)
}
\arguments{
  \item{object}{result of \code{\link{lm}(.)}.}
  \item{c.prob}{coverage probability in \eqn{(0,1)}.}
  \item{confidence}{logical; if true, do (small) confidence band, else,
    realistic prediction band for the mean of \code{k} observations.}
  \item{k}{integer or \code{Inf}; assume \code{k} future observations;
    \code{k = Inf}  corresponds to confidence intervals (for y).}
  \item{col, lty}{attributes for the \code{\link{lines}} to be drawn.}
  \item{do.abline}{logical; if true, the regression line is drawn as well.}
}
\note{
  With \code{\link{predict.lm}(*, interval=)} is available,
  this function \code{linesHyperb.lm} is only slightly more general for
  its \code{k} argument.
}
\author{Martin Maechler, Oct 1995}
\seealso{\code{\link{predict.lm}(*, interval=)} optionally computes
  prediction or confidence intervals.}
\examples{
data(swiss)
      plot(Fertility ~ Education, data = swiss) # the data
(lmS <- lm(Fertility ~ Education, data = swiss))
linesHyperb.lm(lmS)
linesHyperb.lm(lmS, conf=TRUE, col="blue")
}
\keyword{regression}
\keyword{aplot}

\eof
%%-- Original from Vincent Carey, see (commented) E-mail at end !
\name{mat2tex}
\alias{mat2tex}
\title{Produce LaTeX commands to print a matrix}
\usage{
mat2tex(x, file="mat.tex", append=TRUE, digits=3, title)
}
\arguments{
  \item{x}{a matrix}
  \item{file}{names the file to which LaTeX commands should be written}
  \item{append}{logical; if \code{FALSE}, will destroy the file
    \code{file} before writing commands to it; otherwise (by default),
    simply adds commands at the end of file \code{file}.} 
  \item{digits}{integer; setting of \code{\link{options}(digits=..)} for
    purpose of number representation.} 
  \item{title}{a string, possibly using LaTeX commands, which will span
    the columns of the LaTeX matrix}
}
\description{
  ``Translate'' an \R matrix (like object) into a LaTeX table,
  using \code{\\begin{tabular} ...}.
}
\value{
  No value is returned.  This function, when used correctly, 
  only writes LaTeX commands to a file.
}
\author{For S: Vincent Carey \email{vjcarey@sphunix.sph.jhu.edu}, from a
  post on Feb.19, 1991 to S-news.  Port to \R (and a bit more) by
  Martin Maechler \email{maechler@stat.math.ethz.ch}. 
}
\examples{
mex <- matrix(c(pi,pi/2,pi/4,exp(1),exp(2),exp(3)),nrow=2, byrow=TRUE,
               dimnames = list(c("$\\pi$","$e$"),c("a","b","c")))
mat2tex( mex, title="$\\pi, e$, etc." )

## The last command produces the file "mat.tex" containing

##>  \\begin{tabular} {| l|| c| c| c|}  
##>  \\multicolumn{ 4 }{c}{ $\\pi, e$, etc. } \\\\ \\hline 
##>     \\  & a & b & c \\\\ \\hline \\hline 
##>  $\pi$ & 3.14 & 1.57 & 0.785 \\\\ \\hline 
##>  $e$ & 2.72 & 7.39 & 20.1 \\\\ \\hline 
##>  \\end{tabular}  

## Now you have to properly embed the contents of this file
## in a LaTeX document -- for example, you will need a
## preamble, the \\begin{document} statement, etc.

## Note that the backslash needs protection in dimnames
## or title actions.
}
\keyword{interface} % to latex
\keyword{utilities}
%% 
%% From: vjcarey@sphunix.sph.jhu.edu (Vincent J. Carey)
%% Subject: mat2tex, a report generation aid (LONG)
%% To: s-news@stat.wisc.edu
%% Date: Tue, 19 Feb 91 19:25:05 EST
%% 
%% Has there been any work on "report generation" from S
%% beyond the tbl() function?  I have been manually
%% transcribing S matrix elements to LaTeX tables
%% and find this to be a tedious and error-prone process.
%% Certainly there are ways of cutting and pasting window
%% contents to reduce the troubles associated with digit
%% keying, but this is far from foolproof, and is also
%% pretty dull.
%% 
%% Therefore I offer this function, mat2tex(), which
%% produces LaTeX commands to format the entries of
%% an S matrix.  The function will, by default, place
%% the commands in the file "mat.tex".
%% 
%% The documentation file included is fairly explicit. The
%% function does not produce a complete LaTeX program, but
%% a fragment "containing" a matrix.  Column and row labels
%% (which may include references to special LaTeX symbols,
%% provided backslash protection is maintained) are propagated
%% to the LaTeX fragment.  There is a provision for a matrix "title".
%% 
%% Unadulterated output of a simple example (see the doc page
%% for the input):
%% 
%% \begin{tabular} {| l|| c| c| c|}  
%% \multicolumn{ 4 }{c}{ $\pi, e$, etc. } \\ \hline 
%%    \  & a & b & c \\ \hline \hline 
%% $\pi$ & 3.14 & 1.57 & 0.785 \\ \hline 
%% $e$ & 2.72 & 7.39 & 20.1 \\ \hline 
%% \end{tabular}  
%% 
%% Further work needed: digit/decimal-point alignment,
%% consistent boxing of the title, more arguments to allow
%% sensible customization -- e.g., the user may not desire
%% \hline between rows.  Comments, criticism and enhancements 
%% are welcome.
%% 
%% -------------------------------------------------------------------------
%% 
%% Vincent J. Carey
%% Department of Biostatistics
%% Johns Hopkins School of Public Health
%% 
%% vjcarey@sphunix.sph.jhu.edu

\eof
\name{mpl}
\alias{mpl}
\alias{p.m}
\title{Simple Matrix Plots}
\description{
  Do simple matrix plots, providing an easy interface to
  \code{\link{matplot}} by using a default x variable.
}
\usage{
mpl(mat, \dots)
p.m(mat, \dots)
}
\arguments{
  \item{mat}{numeric matrix.}
  \item{\dots}{further arguments passed to \code{\link{matplot}},
    e.g., \code{type}, \code{xlab}, etc.}
}
\details{
  \code{p.m(m)} use the first column of \code{m} as \eqn{x} variable,
  whereas \code{mpl(m)} uses the integers 1, 2, \ldots, \code{nrow(m)}
  as coordinates and \code{rownames(m)} as axis labels if possible.
}
\note{These were really created for playing around with curves etc, and
  probably should be \emph{deprecated} since in concrete examples, using
  \code{matplot()} directly is more appropriate.
}
\author{Martin Maechler}
\seealso{\code{\link{matplot}},
  \code{\link{plot.mts}(*, plot.type = "single")}.}
\examples{
data(animals, package = "cluster")
mpl(animals, type = "l")
}
\keyword{hplot}
\keyword{array}

\eof
\name{mult.fig}
\alias{mult.fig}
\title{Plot Setup for MULTiple FIGures, incl. Main Title}
\description{
  Easy Setup for plotting multiple figures (in a rectangular layout) on
  one page.  It allows to specify a main title and uses \emph{smart}
  defaults for several \code{\link{par}} calls.
}
\usage{
mult.fig(nr.plots, mfrow, mfcol,
         marP = rep(0, 4), mgp = c(1.5, 0.6, 0),
         mar = marP + 0.1 + c(4,4,2,1), oma = c(0,0, tit.wid, 0),
         main = NULL,
         tit.wid = if (is.null(main)) 0 else 1 + 1.5*cex.main,
         quiet = .Device == "postscript",
         cex.main = par("cex.main"), line.main = cex.main - 1/2,
         col.main = par("col.main"), font.main = par("font.main"), \dots)
}
\arguments{
 \item{nr.plots}{integer; the number of plot figures you'll want to draw.}
 \item{mfrow}{\emph{instead} of \code{nr.plots}: integer(2) vector
   giving the rectangular figure layout for \code{\link{par}(mfrow= .)}.}
 \item{mfcol}{\emph{instead} of \code{nr.plots}: integer(2) vector
   giving the rectangular figure layout for \code{\link{par}(mfcol= .)}.}
 \item{marP}{numeric(4) vector of figure margins to \emph{add}
   (``\bold{P}lus'') to default \code{mar}, see below.}
 \item{mgp}{argument for \code{\link{par}(mpg= .)} with a smaller
   default than usual.}
 \item{mar}{argument for \code{\link{par}(mar= .)} with a smaller
   default than usual, using the \code{marP} argument, see above.}
 \item{oma}{argument for \code{\link{par}(oma= .)}, by default for
   adding space for the \code{main} title if necessary.}
 \item{main}{character.  The main title to be used for the whole graphic.}
 \item{tit.wid}{numeric specifying the vertical width to be used for the
   main title; note that this is only used for the default value of
   \code{oma} (s. above).}
 \item{cex.main}{numeric; the character size to be used for the main title.}
 \item{line.main}{numeric; the line at which the title.}
 \item{col.main, font.main}{color and font for main title; see
   \code{\link{par}(*)} for details.}
 \item{quiet}{logical; if \code{TRUE}, do \emph{not} write the reminder
   about resetting \code{\link{par}}.}
 \item{\dots}{Further arguments to \code{\link{mtext}} for the main title.}
}
\value{
  A \code{\link{list}} with two components that are lists themselves, a
  subset of \code{\link{par}()},
  \item{new.par}{the current \code{par} settings.}
  \item{old.par}{the \code{par} \emph{before} the call.}
}
\author{Martin Maechler, UW Seattle, 1990.}
\seealso{\code{\link{par}}, \code{\link{layout}}.}
\examples{
opl <- mult.fig(5, main= expression("Sine Functions " * sin(n * pi * x)))
x <- seq(0, 1, len = 201)
for (n in 1:5)
  plot(x, sin(n * pi * x), ylab ="", main = paste("n = ",n))
par(opl$old.par)

rr <- mult.fig(mfrow=c(5,1), main= "Cosinus Funktionen", cex = 1.5,
               marP = - c(0, 1, 2, 0))
for (n in 1:5)
  plot(x, cos(n * pi * x), type = 'l', col="red", ylab ="")
str(rr)
par(rr$old.par)
## The *restored* par settings:
str(do.call("par", as.list(names(rr$new.par))))
}
\keyword{hplot}

\eof
\name{n.code}
\alias{n.code}
\alias{code2n}
\title{Convert "Round" Integers to Short Strings and Back}
\description{
  \code{n.code} convert ``round integers'' to short character strings.
  This is useful to build up variable names in simulations, e.g.

  \code{code2n} is the \emph{inverse} function of \code{n.code()}.}
}
\usage{
n.code(n,    ndig = 1, dec.codes = c("", "d", "c", "k"))
code2n(ncod, ndig = 1, dec.codes = c("", "d", "c", "k"))
}
\arguments{
  \item{n}{integer vector.}
  \item{ncod}{character vector, typically resulting from
    \code{\link{n.code}}.}
  \item{ndig}{integer giving number of digits before the coding character.}
  \item{dec.codes}{character code for 1, 10, 100, 1000 (etc).}
}
% \details{
%   ~~ If necessary, more details than the __description__  above ~~
% }
\value{
  \code{n.code(n)} returns a \code{\link{character}} vector of the same
  length as \code{n}.

  \code{code2n(ncod)} returns a \code{\link{integer}} vector of the same
  length as \code{ncod}.

  Usually, \code{code2n(n.code(n)) == n}.
}
\author{Martin Maechler}
%\seealso{ ..}
\examples{
n10 <-  c(10,20,90, 100,500, 2000,10000)
(c10 <- n.code(n10))#-> "1d" "2d" "9d" "1c" ..
stopifnot(code2n(c10) == n10)
}
\keyword{utilities}


\eof
\name{n.plot}
\alias{n.plot}
\title{Name Plot: Names or Numbers instead of Points in Plot}
\description{
  A utility function which basically calls \code{\link{plot}(*, type="n")}
  and \code{\link{text}}.   To have names or numbers instead of points
  in a plot is useful for identifaction, e.g., in a residual plot, see
  also \code{\link{TA.plot}}.
}
\usage{
n.plot(x, y = NULL, nam = NULL, abbr = n >= 20 || max(nchar(nam))>=8,
       xlab = NULL, ylab = NULL, log = "",
       cex = par("cex"), col = par("col"), \dots)
}
\arguments{
  \item{x,y}{coordinates at which to plot. If \code{y} is missing,
    \code{x} is used for both, if it's a \code{\link{data.frame},
      \link{list}}, 2-column matrix etc -- via \code{\link{xy.coords}};
    formula do \bold{not} work.}
  \item{nam}{the labels to plot at each (x,y).  Per default, these
    taken from the data \code{x} and \code{y};  case numbers \code{1:n}
    are taken if no names are available.}
  \item{abbr}{logical indicating if the \code{nam} labels should be
    abbreviated -- with a sensible default.}
  \item{xlab,ylab}{labels for the x- and y- axis, the latter being empty
    by default.}
  \item{log}{character specifying if log scaled axes should be used, see
    \code{\link{plot.default}}.}
  \item{cex}{plotting character expansion, see \code{\link{par}}.}
  \item{col}{color to use for \code{\link{text}()}.}
  \item{\dots}{further arguments to be passed to the \code{\link{plot}} call.}
}
\value{
  invisibly, a character vector with the labels used.
}
\author{Martin Maechler, since 1992}
\seealso{\code{\link{plot.default}}, \code{\link{text}}.}
\examples{
 n.plot(1:20, cumsum(rnorm(20)))
 data(cars)
 with(cars, n.plot(speed, dist, cex = 0.8, col = "forest green"))
}
\keyword{hplot}

\eof
\name{nr.sign.chg}
\alias{nr.sign.chg}
\title{Number of Sign Changes in Sequence}
\description{
  Compute the number of sign changes in the sequence \code{y}.
}
\usage{
nr.sign.chg(y)
}
\arguments{
  \item{y}{numeric vector.}
}
\value{
  an integer giving the number of sign changes in sequence \code{y}.
  Note that going from positive to 0 to positive is \emph{not} a sign change.
}
\author{Martin Maechler, 17 Feb 1993.}
\examples{
(y <- c(1:2,1:-1,0:-2))
nr.sign.chg(y)## = 1
}
\keyword{arith}

\eof
\name{p.arrows}
\alias{p.arrows}
\title{Prettified Arrows Plots}
\description{
  Draws arrows, like the \code{\link{arrows}} function, but with
  ``nice'' \emph{filled} arrow heads.
}
\usage{
p.arrows(x1, y1, x2, y2, size = 1, width, fill = 2, ...)
}
\arguments{
  \item{x1, y1}{coordinates of points \bold{from} which to draw.}
  \item{x2, y2}{coordinates of points \bold{to} which to draw.}
  \item{size}{symbol size as a fraction of a character height; default 1.}
  \item{width}{width of the arrow head; defaults to ....}%fixme (code => ??)
  \item{fill}{color for filling the arrow head.}
  \item{\dots}{further arguments passed to \code{\link{segments}()}.}
}
\author{Andreas Ruckstuhl, 19 May 1994; (cosmetic by MM).}
\seealso{\code{\link{arrows}}.}
\examples{
example(arrows, echo = FALSE) #-> x, y, s
plot(x,y, main="p.arrows(.)")
p.arrows(x[s], y[s], x[s+1], y[s+1], col= 1:3, fill = "dark blue")
}
\keyword{aplot}


\eof
\name{p.datum}
\alias{p.datum}
\title{Plot `Datum' (deutsch!) unten rechts}
\description{
  Plot the date (and time, if required) in German, at the lower right hand
  margin of your plot.date
}
\usage{
p.datum(outer = FALSE, cex = 0.75, ...)
}
\arguments{
  \item{outer}{logical; passed to \code{\link{mtext}}.}
  \item{cex}{non-negative; passed to \code{\link{mtext}}.}
  \item{\dots}{any arguments to \code{\link{u.Datumvonheute}}.}
}
\seealso{\code{\link{u.date}}, \code{\link{date}}.}
\examples{
plot(1)
p.datum()
}
\keyword{hplot}%-- one or more ...

\eof
\name{p.dnorm}
\alias{p.dnorm}
\alias{p.dchisq}
\alias{p.dgamma}
\title{Plot Parametric Density Functions}
\description{
  These are utilities for pretty plotting of often used parametric
  densities.
}
\usage{
p.dnorm (mu = 0, s = 1, h0.col = "light gray",
         ms.lines = TRUE, ms.col = "gray", ...)
p.dchisq(nu,    h0.col = "light gray", ...)
p.dgamma(shape, h0.col = "light gray", ...)
}
\arguments{
  \item{mu,s}{numbers, the mean and standard deviation of the normal
    distribution.}
  \item{nu}{positive number, the degrees of freedom \code{df} argument
    for the \eqn{\chi^2}{chi^2}-density function \code{\link{dchisq}}.}
  \item{shape}{number, the \code{shape} parameter for the Gamma
    distribution.}
  \item{h0.col}{color specification for the line \eqn{y = 0}.}
  \item{ms.lines}{logical, used for the normal only: should lines be
    drawn at the mean and \eqn{\pm}{+-} 1 standard deviation.}
  \item{ms.col}{color for the \code{ms} lines if \code{ms.lines} is TRUE.}
  \item{\dots}{further parameter passed to \code{\link{curve}()}, e.g.,
    \code{add = TRUE} for adding to current plot.}
}
\author{Werner Stahel et al.}
\seealso{the underlying density functions,
  \code{\link{dnorm}},  \code{\link{dchisq}},  \code{\link{dgamma}}.}
\examples{
p.dnorm()
p.dnorm(mu=1.5, add = TRUE, ms.lines = FALSE) # add to the plot above

p.dchisq(2, main= "Chi^2 Densities -- nu = 2,3,4")
p.dchisq(3, add = TRUE, col = "red")
p.dchisq(4, add = TRUE, col = "blue")

op <- par(mfrow = c(2,2), mgp = c(1.6, 0.6,0), mar = c(3,3,1,1))
for(sh in 1:4)
   p.dgamma(sh)
par(op)
}
\keyword{hplot}
\keyword{utilities}

\eof
\name{p.hboxp}
\alias{p.hboxp}
\title{Add a Horizontal Boxplot to the Current Plot}
\description{
  Add a horizontal boxplot to the current plot.  This is mainly an
  auxiliary function for \code{\link{hist.bxp}}, since
  \code{\link{boxplot}(*, horizontal = TRUE, add = TRUE)} is usually
  much preferable to this.
}
\usage{
p.hboxp(x, y.lo, y.hi, boxcol = 3,
        medcol = 0, medlwd = 5, whisklty = 2, staplelty = 1)
}
\arguments{
  \item{x}{univariate data set.}
  \item{y.lo, y.hi}{minimal and maximal \emph{user} coordinates
    \bold{or} \code{y.lo = c(ylo,hyi)}.}
  \item{boxcol, medcol}{color of the box and the median line.}
  \item{medlwd}{line width of median line.}
  \item{whisklty, staplelty}{line types of the whisker and the staple,
    the latter being used for the outmost non-outliers.}
}
\details{
....
}
\author{Martin Maechler building on code from Markus and Christian Keller.}
\seealso{\code{\link{boxplot}(**, horizontal = TRUE, add= TRUE)}.}
\examples{
%% FIXME: add!
  ## ==>  See code in 'hist.bxp' (.)
  ##
}
\keyword{aplot}

\eof
\name{p.profileTraces}
\alias{p.profileTraces}
\title{Plot a profile.nls Object With Profile Traces}
\description{
  Displays a series of plots of the profile t function and the likelihood
  profile traces for the parameters in a nonlinear regression model that
  has been fitted with \code{\link{nls}} and profiled with
  \code{\link{profile.nls}}.
}
\usage{
p.profileTraces(x, cex = 1, subtitle = "t-Profil-Plot und Profilspuren")
}
\arguments{
  \item{x}{an object of class \code{nls}, typically resulting from
    \code{\link{nls}(.)}.}
  \item{cex}{character expansion, see \code{\link{par}(cex =)}.}
  \item{subtitle}{a subtitle to set for the plot.}
}
%- \details{
% .........
%- }
\author{Andreas Ruckstuhl, \R port by Isabelle Flckiger and Marcel Wolbers}
\note{the \pkg{stats}-internal \code{stats:::plot.profile.nls} plot
  method just does \dQuote{the diagonals}.
}
\seealso{\code{\link{profile}}, and \code{\link{nls}} (which has
  unexported \code{profile} and \code{stats:::plot.profile.nls} methods).
}
\examples{
require(stats)
data(Puromycin)
Treat <- Puromycin[Puromycin$state == "treated", ]
fm <- nls(rate ~ T1*conc/(T2+conc), data=Treat,
          start = list(T1=207,T2=0.06))
(pr <- profile(fm)) # quite a few things..
op <- par(mfcol=1:2)
plot(pr) # -> 2 `standard' plots
par(op)
## ours:
p.profileTraces(pr)
}
\keyword{hplot}
\keyword{nonlinear}

\eof
\name{p.res.2fact}
\alias{p.res.2fact}
\title{Plot Numeric (e.g. Residuals) vs 2 Factors Using Boxplots}
\description{
  Plots a numeric (``residual like'') variable against two factor
  covariates, using boxplots.
}
\usage{
p.res.2fact(x, y, z, restricted, notch = FALSE,
            xlab = NULL, ylab = NULL, main = NULL)
}
\arguments{
  \item{x,y}{two factors or numeric vectors giving the levels of factors.}
  \item{z}{numeric vector of same length as \code{x} and \code{y},
    typically residuals.}
  \item{restricted}{positive value which truncates the size.  The
    corresponding symbols are marked by stars.}
  \item{notch}{logical indicating if the boxplots should be notched, see
    \code{\link{boxplot}(*,notch)}.}
  \item{xlab,ylab}{axis labels, see \code{\link{plot.default}}, per
    default the actual argument expressions.}
  \item{main}{main title passed to \code{plot}, defaulting to the
    deparsed \code{z} argument.}
}
\details{
  if values \emph{are} restricted, this make use of the auxiliar
  function \code{\link{u.boxplot.x}}.
}
\author{Lorenz Gygax \email{logyg@wild.unizh.ch} and Martin Maechler, Jan.95;
  starting from \code{\link{p.res.2x}()}.
}
\seealso{\code{\link{p.res.2x}}, \code{\link{boxplot}},
  \code{\link{plot.lm}}, \code{\link{TA.plot}}.
}
\examples{
I <- 8; J <- 3; K <- 20
xx <- factor(rep(rep(1:I, rep(K,I)),J))
yy <- factor(rep(1:J, rep(I*K,J)))
zz <- rt(I*J*K, df=5) #-- Student t with 5 d.f.
p.res.2fact(xx,yy,zz, restr= 4, main= "i.i.d. t <- 5 random  |.| <= 4")
mtext("p.res.2fact(xx,yy,zz, restr= 4, ..)",
      line=2, adj=1, outer=TRUE, cex=1)

## Real data
data(warpbreaks)
(fm1 <- lm(breaks ~ wool*tension, data = warpbreaks))
with(warpbreaks,
     p.res.2fact(wool, tension, residuals(fm1)))
}
\keyword{hplot}
\keyword{regression}

\eof
\name{p.res.2x}
\alias{p.res.2x}
\title{Stahel's Residual Plot against 2 X's}
\description{
  Plot Residuals (e.g., of a multiple linear regression)
}
\usage{
p.res.2x(x, y, z, restricted, size = 1, slwd = 1, scol = 2:3,
         xlab = NULL, ylab = NULL, main = NULL,
         xlim = range(x), ylim = range(y), \dots)
}
\arguments{
  \item{x,y}{numeric vectors of the same length specifying 2 covariates.}
  \item{z}{numeric vector of same length as \code{x} and \code{y},
    typically residuals.}
  \item{restricted}{positive value which truncates the size.  The
    corresponding symbols are marked by stars.}
  \item{size}{the symbols are scaled so that \code{size} is the size of
    the largest symbol in cm.}
  \item{slwd, scol}{line width and color(s) for the residual
    \code{\link{segments}}.  If \code{scol} has length 2 as per default,
    the two colors are used for positive and negative \code{z} values,
    respectively.}
  \item{xlab, ylab, main}{axis labels, and title see \code{\link{title}},
    each with a sensible default.  To suppress, use, e.g., \code{main = ""}.}
  \item{xlim, ylim}{the basic x- and y- axis extents, see
    \code{\link{plot.default}}.  Note that these will be slightly
    extended such that segments are not cut off.}
  \item{\dots}{further arguments passed to \code{plot}.}
}
\details{
  ...........
  ..........
}
\references{Stahel, W. (1996) ........}
\author{Andreas Ruckstuhl in June 1991 and
  Martin Maechler, in 1992, '94, 2003-4.}
\seealso{\code{\link{p.res.2fact}}, \code{\link{plot.lm}},
   \code{\link{TA.plot}}.}

\examples{
xx <- rep(1:10,7)
yy <- rep(1:7, rep(10,7))
zz <- rnorm(70)
p.res.2x(xx,yy,zz, restr = 2, main = "i.i.d.  N(0,1) random residuals")
\dontshow{
 p.res.2x(xx,yy,zz, restr = 2, main = "p.res.2x(*, xlim, ylim)",
          xlim = c(2,8), ylim = c(1,4))
}
example(lm.influence, echo = FALSE)
mult.fig(2, marP=c(0,-1,-2,0), main="p.res.2x(*,*, residuals(lm.SR))")
with(LifeCycleSavings,
     { p.res.2x(pop15, ddpi, residuals(lm.SR), scol=c("red", "blue"))
       p.res.2x(pop75, dpi,  residuals(lm.SR), scol=2:1)
     })
}
\keyword{hplot}
\keyword{regression}

\eof
\name{p.scales}
\alias{p.scales}
\title{Conversion between plotting scales: usr, cm, symbol}
\description{
  Give scale conversion factors of three coordinate systems in use for
  traditional R graphics: use, cm, symbol.}
}
\usage{
p.scales(unit = relsysize * 2.54 * min(pin), relsysize = 0.05)
}
\arguments{
  \item{unit}{length of unit (or x and y units) of symbol coordinates in cm.}
  \item{relsysize}{same, as a proportion of the plotting area.}
}
\value{
  A numeric 2x2 matrix, with rows named \code{x} and \code{y}, and
  columns, named \code{"sy2usr"} and \code{"usr2cm"} which give the
  scale conversion factors from \sQuote{symbol} (as given) to
  \sQuote{usr} coordinates and from these to \sQuote{cm}, respectively.
}
\author{Werner Stahel, 1990;  simplification: M.Maechler, 1993, 2004}
\seealso{\code{\link{par}("usr")}, of also \code{("pin")} on which this
    is based.}
\examples{
p.scales()
}
\keyword{dplot}

\eof
\name{p.tachoPlot}
\alias{p.tachoPlot}
\title{Draw Symbol on a Plot}
\description{
Puts a symbol (pointer) on a plot at each of the specified locations.
}
\usage{
p.tachoPlot(x, y, z, angle=c(pi/4,3*pi/4), size,
	    method = c("robust", "sensitive", "rank"),
	    legend = TRUE, show.method = legend,
	    xlab = deparse(substitute(x)), ylab = deparse(substitute(y)),
	    xlim, ylim, \dots)
}
\arguments{
  \item{x,y,z}{coordinates of points. Numeric vectors of the same length.
    Missing values (\code{NA}s) are allowed.}
  \item{angle}{numeric vector whose elements give the angles between the
    horizontal baseline and the minimum and maximum direction of the
    pointer measured clockwise in radians.}
  \item{size}{length of the pointers in cm.}
  \item{method}{string specifying the method to calculate the angle of
    the pointer.  One of \code{"sensitive"}, \code{"robust"} or
    \code{"rank"}.  Only the first two characters are necessary.

    The minimum and maximum direction of the pointer corresponds to
    min(z) and max(z) if method is \code{"sensitive"} or \code{"rank"}
    and to the upper and lower extreme of z if method is \code{"robust"}
    (see \code{boxplot} or \code{rrange} for details).  The angle is
    proportional to z or rank(z) in case of \code{method="rank"}.
  }
  \item{legend}{logical flag: if \code{TRUE} (default), a legend giving
    the values of the minimum and maximum direction of the pointer is drawn.}
  \item{show.method}{logical flag, defaulting to \code{legend}; if true,
    the method name is printed.}
  \item{xlab,ylab}{labels for x and y axis; defaults to the `expression'
    used in the function call.}
  \item{xlim,ylim}{numeric of length 2, the limits for the x and y axis,
    respectively; see \code{\link{plot.default}}.}
  \item{\dots}{further arguments to \code{\link{plot}}.  Graphical
    parameters (see \code{\link{par}}) may also be supplied as arguments
    to this function.}
}
\section{Side Effects}{
  A plot is created on the current graphics device.
}
\details{
  A scatter plot of the variables x and y is plotted. The value of the third
  variable z is given by the direction of a pointer (similar to a
  tachometer). Observations whose z-coordinate is missing are marked by a dot.
}
\author{
  Christian Keller \email{keller@stat.math.ethz.ch}, June 1995
}
\seealso{\code{\link{symbols}}}
\examples{
data(state)
data(USArrests)
p.tachoPlot(state.center $x, state.center $y, USArrests[,"UrbanPop"])

data(mtcars)
par(mfrow=c(2,2))
## see the difference between the three methods (not much differ. here!)
%% hence, IMPROVE the example !
p.tachoPlot(mtcars$hp, mtcars$disp, mtcars$mpg, method="sens")
p.tachoPlot(mtcars$hp, mtcars$disp, mtcars$mpg, method="rank")
p.tachoPlot(mtcars$hp, mtcars$disp, mtcars$mpg, method="rob")
}
\keyword{hplot}

\eof
\name{p.ts}
\alias{p.ts}
\title{plot.ts with multi-plots and Auto-Title -- on 1 page}
\description{
  For longer time-series, it is sometimes important to spread the
  time-series plots over several subplots.
  p.ts(.) does this both automatically, and under manual control.

  Actually, this is a generalization of \code{\link[base]{plot.ts}}
  (with different defaults).
}
\usage{
p.ts(x, nrplots = max(1, min(8, n \%/\% 400)), overlap = nk \%/\% 16,
     date.x = NULL, do.x.axis = !is.null(date.x), do.x.rug = FALSE,
     ax.format, main.tit = NULL, ylim = NULL, ylab = "", xlab = "Time",
     quiet = FALSE, mgp = c(1.25, .5, 0), \dots)
}
\arguments{
  \item{x}{timeseries (possibly multivariate) or numeric vector.}
  \item{nrplots}{number of sub-plots.  Default: in \{1..8\},
    approximately \code{n/400} if possible.}
  \item{overlap}{by how much should subsequent plots overlap. Defaults
    to about 1/16 of sub-length on each side.}
  \item{date.x}{a time \dQuote{vector} of the same length as \code{x}
    and coercable to class \code{"POSIXct"} (see \link{DateTimeClasses}).}
  \item{do.x.axis}{logical specifying if an x axis should be drawn
    (i.e., tick marks and labels).}
  \item{do.x.rug}{logical specifying if \code{\link{rug}} of
    \code{date.x} values should drawn along the x axis.}
  \item{ax.format}{when \code{do.x.axis} is true, specify the
    \code{format} to be used in the call to \code{\link{axis.POSIXct}}.}
  \item{main.tit}{\bold{Main} title (over all plots).  Defaults to name
    of \code{x}.}
  \item{ylim}{numeric(2) or NULL; if the former, specifying the y-range
    for the plots. Defaults to a common pretty range.}
  \item{ylab, xlab}{labels for y- and x-axis respectively, see
    description in \code{\link{plot.default}}.}
  \item{quiet}{logical; if \code{TRUE}, there's no reporting on each subplot.}
  \item{mgp}{numeric(3) to be passed to \code{\link{mult.fig}()}, see
    \code{\link{par}(mgp = .)}.}
  \item{\dots}{further graphic parameters for each \code{\link{plot.ts}(..)}.}
}
\section{Side Effects}{
  A page of \code{nrplots} subplots is drawn on the current
  graphics device.
}
\author{Martin Maechler, \email{maechler@stat.math.ethz.ch}; July 1994
  (for S).}
}
\seealso{\code{p.ts()} calls \code{\link{mult.fig}()} for setup.
  Further, \code{\link{plot.ts}} and \code{\link{plot}}.
}
\examples{
stopifnot(require(stats))
## stopifnot(require(datasets))

data(sunspots)
p.ts(sunspots, nr=1) # == usual  plot.ts(..)
p.ts(sunspots)
p.ts(sunspots, nr=3, col=2)

data(EuStockMarkets)
p.ts(EuStockMarkets[,"SMI"])
## multivariate :
p.ts(log10(EuStockMarkets), col = 2:5)

## with Date - x-axis (dense random dates):
set.seed(12)
x <- as.Date("2000-02-29") + cumsum(1+ rpois(1000, lambda= 2.5))
z <- cumsum(.1 + 2*rt(1000, df=3))
p.ts(z, 4, date.x = x)
p.ts(z, 6, date.x = x, ax.format = "\%b \%Y", do.x.rug = TRUE)
}
\keyword{hplot}
\keyword{ts}

\eof
\name{paste.vec}
\alias{paste.vec}
\title{Utility for 'Showing' S vectors}
\description{
  A simple utility for displaying simple S vectors;
  can be used as debugging utility.
}
\usage{
paste.vec(name, digits = options()$digits)
}
\arguments{
  \item{name}{string with an variable name which must exist in the
    current environment (\R session).}
  \item{digits}{how many decimal digits to be used; passed to
    \code{\link{format}}.}
}
\value{
  a string of the form "NAME =  x1 x2 ..."
}
\author{Martin Maechler, about 1992.}
\examples{
 x <- 1:4
 paste.vec(x)   ##->  "x = 1 2 3 4"
}
\keyword{utilities}

\eof
\name{pl.ds}
\alias{pl.ds}
\title{Plot Data and Smoother / Fitted Values}
\description{
  For one-dimensional nonparametric regression, plot the data and fitted
  values, typically a smooth function, and optionally use segments to
  visualize the residuals.
}
\usage{
pl.ds(x, yd, ys, xlab = "", ylab = "", ylim = rrange(c(yd, ys)),
      xpd = TRUE, do.seg = TRUE, seg.p = 0.95,
      segP = list(lty = 2, lwd = 1,   col = 2),
      linP = list(lty = 1, lwd = 2.5, col = 3),
      \dots)
}
\arguments{
  \item{x, yd, ys}{numeric vectors all of the same length, representing
    \eqn{(x_i, y_i)} and fitted (smooth) values \eqn{\hat{y}_i}{y^_i}.
    Note that \code{x} will be sort increasingly if necessary, and
    \code{yd} and \code{ys} accordingly.}
  \item{xlab, ylab}{x- and y- axis labels, as in \code{\link{plot.default}}.}
  \item{ylim}{limits of y-axis to be used; defaults to a \emph{robust}
    range of the values.}
  \item{xpd}{see \code{\link{par}(xpd=.)}; by default do allow to draw
    outside the plot region.}
  \item{do.seg}{logical indicating if residual segments should be drawn,
    at \code{x[i]}, from \code{yd[i]} to \code{ys[i]} (approximately,
    see \code{seg.p}).}
  \item{seg.p}{segment percentage of segments to be drawn, from
    \code{yd} to \code{seg.p*ys + (1-seg.p)*yd}.}
  \item{segP}{list with named components \code{lty, lwd, col} specifying
    line type, width and color for the residual segments,
    used only when \code{do.seg} is true.}
  \item{linP}{list with named components \code{lty, lwd, col} specifying
    line type, width and color for \dQuote{smooth curve lines}.}
  \item{\dots}{further arguments passed to \code{\link{plot}}.}
}
\author{Martin Maechler, 1990-1994}
\note{Non-existing components in the lists \code{segP} or \code{linP}
  will result in the \code{\link{par}} defaults to be used.
}
%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
 data(cars)
 x <-  cars$speed
 yd <- cars$dist
 ys <- lowess(x, yd,f = .2)$y
 pl.ds(x, yd, ys)

 ## More interesting : Version of example(Theoph)
 data(Theoph)
 Th4 <- subset(Theoph, Subject == 4)
 fm1 <- nls(conc ~ SSfol(Dose, Time, lKe, lKa, lCl), data = Th4)
 pl.ds(Th4$Time, Th4$conc, fitted(fm1),
       sub  = "Theophylline data - Subject 4 only",
       segP = list(lty=1,col=2), las = 1)
 xvals <- seq(0, par("usr")[2], len = 55)
 lines(xvals, predict(fm1, newdata = list(Time = xvals)), col = 4)
}
\keyword{hplot}

\eof
\name{plotCI}
\alias{plotCI}
\title{Plot Confidence Intervals / Error Bars}
\description{
  THIS IS NOW (2004-08-12) DEPRECATED !!!

  DO USE  \code{\link[gregmisc]{plotCI}} from package \pkg{gregmisc} INSTEAD!


  Given a set of x and y values and upper and lower bounds,
  this function plots the points with error bars.  Can be used for
  confidence interval visualization as well.
}
\usage{
plotCI(x, y = NULL, uiw, liw = uiw, aui = NULL, ali = aui,
       err = "y", xlim = NULL, ylim = NULL, type = "p", log = "",
       sfrac = 0.01, gap = 0, add = FALSE,
       col = par("col"), lwd = par("lwd"), slty = par("lty"),
       scol = col, pt.bg = NA,
       xlab = NULL, ylab = NULL, main = "", axes = TRUE, \dots)
}
\arguments{
  \item{x}{the x coordinates of points in the plot}
  \item{y}{the y coordinates of points in the plot}
  \item{uiw}{the width of the upper portion of the confidence region,
    or (if \code{liw} is missing) the width of both halves of
    the confidence region}
  \item{liw}{the width of the lower portion of the confidence region (if
    missing, the function assumes symmetric confidence bounds)}
  \item{aui}{the absolute upper limit of the confidence region}
  \item{ali}{the absolute lower limit of the confidence region}
  \item{err}{character giving the direction of error bars: \code{"x"}
    for horizontal, \code{"y"} for vertical (\code{"xy"} would be nice
    but is not implemented yet).}
  \item{xlim}{x limits of the plot}
  \item{ylim}{y limits of the plot}
  \item{type}{point/line type; passed to \code{\link{points}}}
  \item{log}{character indicating if log scales should be use in x or y;
    passed to \code{\link{plot.default}()}.}
  \item{gap}{size of gap in error bars around points (default 0)}
  \item{sfrac}{scaling factor for the size of the \code{serifs} (end bars) on
    the confidence bars, in x-axis units}
  \item{add}{logical; if FALSE (default), create a new plot; if TRUE,
    add error bars to an existing plot.}
  \item{col}{color of points}
  \item{lwd}{line width of error bars}
  \item{slty}{line type of error bars}
  \item{scol}{color of error bars}
  \item{pt.bg}{background color of points (use
    \code{pch=21, pt.bg=par("bg")} to get open points superimposed on
    error bars).}
  \item{xlab, ylab}{horizontal and vertical axis labels.}
  \item{main}{main title, see \code{\link{plot.default}} or
    \code{\link{title}}.}
  \item{axes}{logical indicating if axes should drawn, see
    \code{\link{plot.default}}.}
  \item{\dots}{any other parameters to be passed through to
    \code{\link{plot}}.}
}
\value{
None.
}
\author{Ben Bolker \email{bolker@zoo.ufl.edu}
  (documentation and tweaking of a function provided by Bill Venables)
}
\note{
  The package \pkg{gregmisc} has been containing another extension of
  Ben's original posting with the same name
  \code{\link[gregmisc]{plotCI}}.  We should really merging the two
  versions into Greg's one, i.e., eventually deprecate this version (of
  package \pkg{sfsmisc}).
}
\section{TO DO}{Fixes for logarithmic axes; simultaneous horizontal and
  vertical error bars.  The main problem is figuring how parameters
  should be specified.
}
\examples{
  y <- runif(10)
  err <- runif(10)
  plotCI(1:10,y,err)
  plotCI(1:10,y,err,2*err,lwd=2,col="red")
  err.x <- runif(10)
  err.y <- runif(10)
  plotCI(1:10,y,err.y,pt.bg=par("bg"),pch=21)
  plotCI(1:10,y,err.x,pt.bg=par("bg"),pch=21,err="x",add=TRUE)
}
\keyword{hplot}
%% 318 Carr Hall                                bolker@zoo.ufl.edu
%% Zoology Department, University of Florida    http://www.zoo.ufl.edu/bolker
%% Box 118525                                   (ph)  352-392-5697
%% Gainesville, FL 32611-8525                   (fax) 352-392-3704

\eof
% Originally /u/maechler/S/GRAPHICS/plot.step.d,v 1.2 1997/05/27 10:17:27
% Copyright (c), 1991, 1997 Martin Maechler, Statistik, ETH Zurich
%		Martin Maechler <maechler@stat.math.ethz.ch>
\name{plotStep}
\alias{plotStep}
\title{Plot a Step Function}
\description{
  Plots a step function
  f(x)= \eqn{\sum_i  y_i  1_[ t_{i-1}, t_i ](x) }{sum(i; y[i] *
    Ind[t[i-1], t[i])(x))}, i.e., a piecewise constant function of one variable.
  With one argument, plots \bold{the} empirical cumulative distribution
  function.
}
\usage{
plotStep(ti, y,
          cad.lag = TRUE,
          verticals = !cad.lag,
          left.points= cad.lag, right.points= FALSE, end.points= FALSE,
	  add = FALSE,
	  pch = par('pch'),
          xlab=deparse(substitute(ti)), ylab=deparse(substitute(y)),
          main=NULL, \dots)
}
\arguments{
  \item{ti}{numeric vector = \code{X[1:N]} or  \code{t[0:n]}.}
  \item{y}{numeric vector \code{y[1:n]}; if omitted take y = k/N
    for empirical CDF.}
  \item{cad.lag}{logical: Draw 'cad.lag', i.e., ``continue  droite, limite 
    gauche''.  Default = TRUE.}
  \item{verticals}{logical: Draw vertical lines?  Default= ! cad.lag}
  \item{left.points}{logical: Draw left points?     Default= cad.lag}
  \item{right.points}{logical: Draw right points?    Default= FALSE}
  \item{end.points}{logical: Draw 2 end points?    Default= FALSE}
  \item{add}{logical: Add to existing plot?    Default= FALSE}
  \item{pch}{The plotting character for points}
  \item{xlab}{Label of x-axis}
  \item{ylab}{Label of y-axis}
  \item{main}{main title; defaults to the call' if you do not want a title,
    use \code{main = ""}.}
  \item{\dots}{Any valid argument to \code{\link{plot}(..)}.}
}
\value{
[INVISIBLY:]  List with components \code{t} and \code{y}.
}
\section{Side Effects}{
  Calls plot(..), points(..),  segments(..) appropriately
  and plots on current graphics device.
}
\author{
  Martin Maechler, Seminar for Statistics, ETH Zurich,
  \email{maechler@stat.math.ethz.ch}, 1991 ff.
}
\seealso{
  The \code{\link{plot}} methods \code{\link{plot.ecdf}} and
  \code{\link{plot.stepfun}} in \R which are conceptually nicer.

  \code{\link{segments}(\dots, method = "constant")}.
}
\examples{
##-- Draw an Empirical CDF  (and see the default title ..)
plotStep(rnorm(15))

plotStep(runif(25), cad.lag=FALSE)
plotStep(runif(25), cad.lag=FALSE, add=TRUE, lty = 2)

ui <- sort(runif(20))
plotStep(ui, ni <- cumsum(rpois(19, lambda=1.5) - 1.5), cad.lag = FALSE)
plotStep(ui, ni, verticals = TRUE, right.points = TRUE)

plotStep(rnorm(201), pch = '.') #- smaller points
}
\keyword{nonparametric}
\keyword{hplot}

\eof
\name{pmax.sa}
\alias{pmax.sa}
\alias{pmin.sa}
\title{Parallel Maxima / Minima (Scalar, Array)}
\description{
  These are versions of \code{\link{pmax}} and \code{\link{pmin}}
  which return (more-dimensional) arrays.
}
\usage{
pmax.sa(scalar, arr)
pmin.sa(scalar, arr)
}
\arguments{
  \item{scalar}{numeric scalar.}
  \item{arr}{any numeric \R object, typically array.}
}
\value{
  an array with the same \code{\link{dim}} and \code{\link{dimnames}} as
  the input \code{arr}.
}
\author{Martin Maechler}
\seealso{\code{\link{pmin}}.}
\examples{
m <- cbind(a=1:5, b=3:7, c=-2:2)
pmax   (3, m) # < dropping all attributes
pmax.sa(3, m) # matrix with dimnames

## no "added value" for simple case:
x <- rpois(20, 7)
stopifnot(identical(pmin   (2.1, x),
		    pmin.sa(2.1, x)),
	  identical(pmax   (2.1, 3),
		    pmax.sa(2.1, 3)))
}
\keyword{array}
\keyword{utilities}

\eof
\name{polyn.eval}
\alias{polyn.eval}
\title{Evaluate Polynomials}
\description{
  Evaluate one or several univariate polynomials at several locations,
  i.e. compute \code{coef[1] + coef[2]*x + ... + coef[p+1]* x^p}
  (in the simplest case where \code{x} is scalar and \code{coef} a vector).
}
\usage{
polyn.eval(coef, x)
}
\arguments{
  \item{coef}{numeric vector or matrix.  If a vector, \code{x} can be an
    array and the result matches \code{x}.\cr
    If \code{coef} is a matrix it specifies several polynomials of the
    same degree as rows, \code{x} must be a vector, \code{coef[,k]} is
    for \eqn{x^{k-1}}{x^(k-1)} and the result
    is a matrix of dimension \code{length(x) * nrow(coef)}.}
  \item{x}{numeric vector or array.  Either \code{x} or \code{coef} must
    be a vector.}
}
\details{
  The stable ``Horner rule'' is used for evaluation in any case.
}
\value{
  numeric vector or array, depending on input dimensionalities, see above.
}
\author{Martin Maechler, ages ago.}
\seealso{For much more sophisticated handling of polynomials, use the
  \code{polynom} package, e.g. \code{\link[polynom]{predict.polynom}}.
}
\examples{
polyn.eval(c(1,-2,1), x = 0:3)# (x - 1)^2
polyn.eval(c(0, 24, -50, 35, -10, 1), x = matrix(0:5, 2,3))# 5 zeros!
(cf <- rbind(diag(3), c(1,-2,1)))
polyn.eval(cf, 0:5)
}
\keyword{arith}

\eof
\name{posdefify}
\alias{posdefify}
\title{Find a Close Positive Definite Matrix}
\description{
  From a matrix \code{m}, construct a \emph{"close"} positive definite
  one.
}
\usage{
posdefify(m, method = c("someEVadd", "allEVadd"), symmetric, eps.ev = 1e-07)
}
\arguments{
  \item{m}{a numeric (square) matrix.}
  \item{method}{a string specifying the method to apply; can be abbreviated.}
  \item{symmetric}{logical, simply passed to \code{\link{eigen}}.}
  \item{eps.ev}{number specifying the tolerance to use, see Details
    below.}
}
\details{
  We form the eigen decomposition
  \deqn{m = V \Lambda V'}{m = V L V'} where \eqn{\Lambda}{L} is the
  diagonal matrix of eigenvalues, \eqn{\Lambda_{j,j} = \lambda_j}{L[j,j]
    = l[j]}, with \emph{decreasing} eigenvalues \eqn{\lambda_1 \ge
    \lambda_2 \ge \ldots \ge \lambda_n}{l[1] >= l[2] >= ... >= l[n]}.

  When the smallest eigenvalue \eqn{\lambda_n}{l[n]} are less than
  \code{Eps <- eps.ev * abs(lambda[1])}, i.e., negative or \dQuote{almost
    zero}, some or all eigenvalues are replaced by \emph{positive}
  (\code{>= Eps}) values,
  \eqn{\tilde\Lambda_{j,j} = \tilde\lambda_j}{L~[j,j] = l~[j]}.
  Then, \eqn{\tilde m = V \tilde\Lambda V'}{m~ = V L~ V'} is computed
  and rescaled in order to keep the original diagonal (where that is
  \code{>= Eps}).
}
\value{
  a matrix of the same dimensions and the \dQuote{same} diagonal
  (i.e. \code{\link{diag}}) as \code{m} but with the property to
  be positive definite.
}
%\references{ ~put references to the literature/web site here ~ }
\author{Martin Maechler}
\seealso{\code{\link{eigen}} on which the current methods rely.}
\examples{
 set.seed(12)
 m <- matrix(round(rnorm(25),2), 5, 5); m <- 1+ m + t(m); diag(m) <- diag(m) + 4
 m
 posdefify(m)
 1000 * zapsmall(m - posdefify(m))
}
\keyword{algebra}
\keyword{array}

\eof
\name{potatoes}
\alias{potatoes}
\docType{data}
\title{Fisher's Potato Crop Data}
\description{
  %% FIXME
  %% ----
}
\usage{data(potatoes)}
\format{
  A data frame with 64 observations on the following 5 variables.
  \describe{
    \item{pos}{a factor with levels \code{1:4}.}
    \item{treat}{a factor with 16 levels \code{A} to \code{H} and
      \code{J} to \code{Q}, i.e., \code{LETTERS[1:17][-9]}.}
    \item{nitrogen}{a factor specifying the amount of nitrogen
      sulfate (\eqn{NH_4}), with the four levels \code{0,1,2,4}.}
    \item{potash}{a factor specifying the amount of potassium (K,
      \sQuote{kalium}) sulfate, with the four levels \code{0,1,2,4}.} \item{yield}{a numeric vector giving the yield of potatoes in ...}% << FIXME
  }
}
\details{
FIXME %%  more details than the __description__ above ~~
}
\source{
  Bennett, J. H. (1972)
  \emph{Collected Papers of R. A. Fischer} vol.~II, 1925-31;
  The University of Adelaide.
  %% Einer der Blcke:
  %% Stahel "Statist. Datenanalyse" Beisp.Kartoffelertrag (2nd ed. 251 a, 253 h)
}
\references{
  T.Eden and R. A. Fisher (1929)
  Studies in Crop Variation. VI. Experiments on the Response of the
  Potato to Potash and Nitrogen.
  \emph{J. Agricultural Science} \bold{19}, 201--213.
  Accessible from Bennett (1972), see above.
}
\examples{
data(potatoes)
## See the experimental design:
with(potatoes, {
     cat("4 blocks of experiments;",
         "each does every (nitrogen,potash) combination (aka 'treat'ment) once.",
         '', sep="\n")
     print(ftable(table(nitrogen, potash, treat)))
     print(ftable(tt <- table(pos,potash,nitrogen)))
     tt[cbind(pos,potash,nitrogen)] <- as.character(treat)
     cat("The 4 blocks  pos = 1, 2, 3, 4:\n")
     ftable(tt)
     })
## First plot:
with(potatoes, interaction.plot(potash,nitrogen, response=yield))

## ANOVAs:
summary(aov(yield ~ nitrogen * potash + Error(pos), data = potatoes))
    # "==>" can use simply
summary(aov(yield ~ nitrogen + potash + pos, data = potatoes))
    # and
summary(aov(yield ~ nitrogen + potash, data = potatoes))
}
\keyword{datasets}

\eof
\name{printTable2}
% source in ../R/printTable.R
\alias{printTable2}
\alias{margin2table}
\alias{print.margin2table}
%\title{Compute and Print Marginals etc for 2-way Contingency Tables}
\title{Berechne und Drucke Randtotale etc fr 2-weg Kontingenz Tafeln}
\description{
  2-weg Kontingenztafel mit allem zusammen

  \code{margin2table()} computes marginals adds them to the table and
  returns a \code{margin2table} object the print method for which adds
  text decorations (using \code{"-"} and \code{"|"}).
}
\usage{
printTable2(table2, digits = 3)
margin2table(x, totName = "sum")
\method{print}{margin2table}(x, digits = 3, \dots)
}
\arguments{
  \item{table2}{Matrix mit Anzahlen, die Kontingenztafel.}
  \item{x}{a matrix, or the result of \code{margin2table}.}
  \item{digits}{Anzahl Dezimalstellen, auf die die Hufigkeiten gerundet
    werden sollen.}
  \item{totName}{string to use as row- and column- name if \code{x} has
    corresponding \code{\link{dimnames}}.}
  \item{\dots}{further potential arguments, unused currently.}
}
% \details{
%  .........
% }
\value{
  \code{margin2table} returns a matrix with \emph{added marginals},
  i.e., an extra row and column, and is of class \code{margin2table}
  which has a nice print method.

  \code{printTable2} is just producing output.
}
\author{Martin Maechler, Feb.1993; then Dec 2003}
\seealso{\code{\link{table}}, \code{\link{ftable}}.}

\examples{
m <- diag(3); colnames(m) <- letters[1:3]
margin2table(m)
margin2table(m / sum(m))

data(HairEyeColor)
margin2table(HairEyeColor[,, "Male"])
printTable2(HairEyeColor[,, "Male"])
printTable2(HairEyeColor[,, "Female"])
}
\keyword{utilities}


\eof
\name{prt.DEBUG}
\alias{prt.DEBUG}
\title{Utility Printing in DEBUG mode}
\description{
  This function prints out its arguments as \code{\link{cat}()} does,
  additionally printing the name of function in which it's been called ---
  only when a global variable \code{DEBUG} exists and is
  \code{\link{TRUE}}.\cr

  The global \code{DEBUG} has been a cheap precursor to the now standard
  \R's \code{\link{options}(verbose= .)} setting.
}
\usage{
prt.DEBUG(\dots, LEVEL = 1)
}
\arguments{
  \item{\dots}{arguments to be passed to \code{\link{cat}(\dots)} for
    printing.}
  \item{LEVEL}{integer (or logical) indicating a debugging level for printing.}
}
\author{Martin Maechler, originally for S-PLUS.}
\note{This is mainly kept for historical reasons (and old code
  fragments), but sometimes I still consider renaming it and have it
  work using \code{getOption("verbose")} alone.
}
\keyword{debugging}


\eof
\name{ps.end}
\alias{ps.end}
\title{Close PostScript device opened by `ps.do'}
\usage{
ps.end(call.gv= NULL, command = getOption("eps_view"),
       debug = getOption("verbose"))
}
\arguments{
 \item{call.gv}{logical, indicating if ghostview (or the command given
  by \code{command}) should be called.  By default, find out if
  ghostview is already runing on this file and only call it if needed.}
 \item{command}{character, giving a system command for PostScript previewing.
   By default, \code{getOption("eps_view")} is set to\cr
   \code{gv -watch -geometry -0+0 -magstep -2 -media BBox -noantialias}
   which assumes \code{gv} (aka \emph{ghostview}) to be in your OS path.}
 \item{debug}{logical; if \code{TRUE} print information during execution.}
}
\description{
  Closes the PostScript file (\code{\link{postscript}}), openend by a
  previous \code{\link{ps.do}} (or \code{\link{ps.latex}}) call, using
  \code{\link{dev.off}}, and additionally opens a previewer for that
  file, \emph{unless} the previewer is already up.  This almost provides
  an \sQuote{interactive} device (like \code{\link{x11}}) for
  \code{\link{postscript}}.
}
\details{
  Depends heavily on Unix tools, such as \code{ps}.
}
\author{Martin Maechler}
\seealso{\code{\link{postscript}},..}
\examples{
if(interactive() \dontshow{ || Sys.getenv("USER") == "maechler" }
  ) {
    ps.do("ex.ps")
    data(sunspots)
    plot(sunspots)
    ps.end()
  }
}
\keyword{device}

\eof
\name{ps.latex}
\alias{ps.do}
\alias{ps.latex}
\title{PostScript Preview Device with Optional `LaTeX' Touch}
\usage{
ps.do(file, width=-1, height=-1, onefile=FALSE, horizontal=FALSE,
      title = NULL, \dots)

ps.latex(file, height = 5 + main.space * 1.25,  width = 9.5,
         main.space=FALSE, lab.space = main.space,
         paper = "special", title = NULL,
         lab=c(10, 10, 7), mgp.lab=c(1.6, 0.7, 0), mar=c(4, 4, 0.9, 1.1), \dots)
}
\arguments{
  \item{file}{character giving the PostScript file name to be written.}
  \item{height}{device height in \emph{inches}, \code{height * 2.54} are
    \emph{cm}.  The default is 5 plus 1.25 iff \code{main.space}.}
  \item{width}{device width in \emph{inches}; for this and
    \code{height}, see \code{\link{postscript}}.}
  \item{onefile, horizontal}{logicals passed to
    \code{\link{postscript}(..)}, most probably to be left alone.}
  \item{title}{PostScript (not plot!) title passed to
    \code{\link{postscript}()}; by default use a title with \R version
    and \code{file} in it.}
  \item{main.space}{logical; if true, leave space for a main title
    (unusual for LaTeX figures!).}
  \item{lab.space}{logical; if true, leave space for x- and y- labels
    (by \emph{not} subtracting from \code{mar}).}
  \item{paper}{character, see \code{\link{postscript}}.  Only if this is
    \code{"special"} are your choices of \code{width} and \code{height}
    completely honored (and this may lead to files that can't print on
    A4) with resizing.}
  \item{lab}{integer of length 3, \code{lab[1:2]} are desired number of
    tick marks on x- and y- axis, see \code{\link{par}(lab=)}.}
  \item{mgp.lab}{three decreasing numbers determining space for axis
    labeling, see \code{\link{par}(mgp=)}, the default is here smaller
    than usual.}
  \item{mar}{four numbers, indicating marginal space, see
    \code{\link{par}(mar=)}, the default is here smaller than usual.}
  \item{\dots}{arguments passed to \code{\link{ps.do}()} from
    \code{ps.latex}} and to \code{\link{ps.options}} from \code{ps.do}.}
}
\description{
  Both functions start a ``Pseudo PostScript Preview'' device, using
  \code{\link{postscript}}, and further registering the file name for
  subsequent calls to \code{\link{ps.end}}.
}
\details{
  \code{ps.latex} has an additional \LaTeX flavor, and just differs by
  some extra \code{\link{par}} settings: After \code{\link{ps.do}(..)}
  is called, the graphical parameters \code{c("mar", "mgp", "lab")} are
  reset (to values that typically are better than the defaults for LaTeX
  figures).
}
\value{
  A list with components
  \item{old.par}{containing the old \code{par} values}
  \item{new.par}{containing the newly set \code{par} values}
}
\author{Martin Maechler}
\seealso{\code{\link{ps.end}}, \code{\link{postscript}},
  \code{\link{dev.print}}.
}
\examples{
if(interactive()) {
 ps.latex("ps.latex-ex.ps", main= TRUE)
  data(sunspots)
  plot(sunspots,main=paste("Sunspots Data, n=",length(sunspots)),col="red")
 ps.end()

 ps.do("ps_do_ex.ps")
   example(plot.function)
 ps.end()
}
}
\keyword{device}

\eof
\name{quadrant}
\alias{quadrant}
\title{Give the Quadrant Number of Planar Points}
\description{
  Determine the quadrant of planar points, i.e. in which of the four
  parts cut by the x- and y- axis the points lie.  Zero values
  (i.e. points on the axes) are treated as if \emph{positive}.
}
\usage{
quadrant(x, y=NULL)
}
\arguments{
  \item{x,y}{numeric vectors of the same length, or \code{x} is an `x-y'
    structure and \code{y=NULL}, see \code{\link{xy.coords}}.}
}
\value{
  numeric vector of same length as \code{x} (if that's a vector) with
  values in \code{1:4} indicating the quadrant number of the
  corresponding point.
}
%%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
xy <- as.matrix(expand.grid(x= -7:7, y= -7:7)); rownames(xy) <- NULL
(qu <- quadrant(xy))
plot(xy, col = qu+1, main = "quadrant() number", axes = FALSE)
abline(h=0, v=0, col="gray") # the x- and y- axis
text(xy, lab = qu, col = qu+1, adj = c(1.4,0))
}
\keyword{utilities}


\eof
\name{rot2}
\alias{rot2}
\title{Rotate Planar Points by Angle}
\description{
  Rotate planar (xy) points by angle \code{phi} (in radians).
}
\usage{
rot2(xy, phi)
}
\arguments{
  \item{xy}{numeric 2-column matrix, or coercable to one.}
  \item{phi}{numeric scalar, the angle in radians (i.e., \code{phi=pi}
    corresponds to 180 degrees) by which to rotate the points.}
}
\value{
  A two column matrix as \code{xy}, containing the rotated points.
}
\author{Martin Maechler, Oct.1994}
%%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
## Rotate three points by 60 degrees :
(xy0 <- rbind(c(1,0.5), c(1,1), c(0,1)))
(Txy <- rot2(xy0, phi = 60 * pi/180))
plot(xy0, col = 2, type = "b", asp = 1,
     xlim=c(-1,1), ylim=c(0,1.5), main = "rot2(*, pi/3) : 2d rotation by 60")
points(Txy, col = 3, type = "b")
O <- rep(0,2); P2 <- rbind(xy0[2,], Txy[2,])
arrows(O,O,P2[,1],P2[,2], col = "dark gray")

xy0 <- .8*rbind(c(1,0), c(.5,.6), c(.7,1), c(1,1), c(.9,.8), c(1,0)) - 0.2
plot(xy0, col= 2, type="b", main= "rot2( <polygon>, pi/4 * 1:7)", asp=1,
     xlim=c(-1,1),ylim=c(-1,1), lwd= 2, axes = FALSE, xlab="", ylab="")
abline(h=0, v=0, col="thistle"); text(1.05, -.05, "x"); text(-.05,1.05, "y")
for(phi in pi/4 * 0:7)
   do.call("arrows",c(list(0,0),rot2(xy0[2,], phi), length=0.1, col="gray40"))
for(phi in pi/4 * 1:7)
   polygon(rot2(xy0, phi = phi), col = 1+phi/(pi/4), border=2, type = "b")
}
\keyword{manip}
\keyword{math}

\eof
\name{rrange}
\alias{rrange}
\title{Robust Range using Boxplot `Quartiles'}
\description{
  Compute a robust range, i.e. the usual \code{\link{range}()} as long
  as there are no outliers,  using the ``whisker boundaries'' of
  \code{\link{boxplot}}.
}
\usage{
rrange(x, range=1, coef = 1.5, na.rm = TRUE)
}
\arguments{
  \item{x}{numeric vector the robust range of which shall be computed.}
  \item{range}{number for S compatibility; \code{1.5 * range} is
    equivalent to \code{coef}.}
  \item{coef}{numeric multiplication factor definying the outlier
    boundary, see ``Details'' below.}
  \item{na.rm}{logical indicating how \code{\link{NA}} values should be
    handled;  they are simply dropped when \code{na.rm = TRUE} as by default.}
}
\details{
  The robust range is really just what \code{\link{boxplot.stats}(x,
    coef=coef)} returns as the whisker boundaries.
  This is the most extreme values \code{x[j]} still inside median
  plus/minus \code{coef * IQR}.
}
\value{
  numeric vector \code{c(m,M)} with \eqn{m \le M}{m <= M} which is (not
  strictly) inside \code{range(x) = c(min(x),max(x))}.
}
\author{Martin Maechler, 1990.}
\seealso{\code{\link{range}}, \code{\link{fivenum}}, \code{\link{boxplot}}.}
\examples{
stopifnot(rrange(c(1:10,1000)) == c(1,10))
}
\keyword{univar}
\keyword{robust}

\eof
\name{signi}
\alias{signi}
\title{Rounding to Significant Digits}
\description{
  Rounds to significant digits similarly to \code{\link{signif}}.
}
\usage{
signi(x, digits = 6)
}
\arguments{
  \item{x}{numeric vector to be rounded.}
  \item{digits}{number of significant digits required.}
}
\value{
  numeric vector ``close'' to \code{x}, i.e. by at least \code{digits}
  significant digits.
}
\author{Martin Maechler, in prehistoric times (i.e. before 1990).}
\note{
  This is really just \code{round(x, digits - trunc(log10(abs(x))))} and
  hence mainly of didactical use.   Rather use \code{signif()} otherwise.
}
\seealso{\code{\link{signif}}, \code{\link{round}}.}
\examples{
(x1 <- seq(-2, 4, by = 0.5))
identical(x1, signi(x1))# since 0.5 is exact in binary arithmetic
(x2 <- pi - 3 + c(-5,-1,0, .1, .2, 1, 10,100))
signi(x2, 3)
}
\keyword{arith}

\eof
\name{tapplySimpl}
\alias{tapplySimpl}
\title{More simplification in tapply() result}
\description{
  For the case of more than two categories or indices (in \code{INDEX}),
  traditional \code{\link{tapply}(*, simplify = TRUE)} still returns a
  list when an array may seem more useful and natural.  This is provided
  by \code{tapplySimpl()} if the function \code{FUN()} is defined such
  as to return a vector of the same length in all cases.
}
\usage{
tapplySimpl(X, INDEX, FUN, \dots)
}
\arguments{
  \item{X}{an atomic object, typically a vector.  All these arguments
    are as in \code{\link{tapply}()} and are passed to \code{tapply(..)}.}
  \item{INDEX}{list of (typically more than one) factors, each of same
    length as \code{X}.}
  \item{FUN}{the function to be applied.  For the result to be
    simplifiable, \code{FUN()} must return a vector of always the same
    length.}
  \item{\dots}{optional arguments to \code{FUN}.}
}
\value{
  If the above conditions are satisfied, the list returned from
  \code{r <- tapply(X, INDEX, FUN, \dots)} is simplified into an
  \code{\link{array}} of rank \eqn{1 + \#\{indices\}}, i.e.,
  \code{1+length(INDEX)}; otherwise, \code{tapplySimpl()} returns the list
  \code{r}, i.e., the same as \code{tapply()}.
}
\author{Martin Maechler, 14 Jun 1993 (for S-plus).}
\seealso{\code{\link{tapply}(*, simplify=TRUE)}.}
\examples{
  ## Using tapply() would give a list (with dim() of a matrix);
  ## here we get 3-array:

  data(esoph)
  with(esoph, {
       mima <<- tapplySimpl(ncases/ncontrols, list(agegp, alcgp),  range)
       stopifnot(dim(mima) == c(2, nlevels(agegp), nlevels(alcgp)))
       })
  aperm(mima)
}
\keyword{iteration}
\keyword{category}

\eof
\name{tkdensity}
\alias{tkdensity}
\title{GUI Density Estimation using Tcl/Tk}
\description{
  This is graphical user interface (GUI) to \code{\link{density}},
  allowing for dynamic bandwidth choice and a simple kind of zooming,
  relying on \code{library(tcltk)}.
}
\usage{
tkdensity(y, n = 1024, log.bw = TRUE, showvalue = TRUE,
          xlim = NULL, do.rug = size < 1000,
          from.f = if (log.bw) -2 else 1/1000,
          to.f   = if (log.bw) +2.2 else 2,
          col = 2)
}
\arguments{
  \item{y}{numeric; the data the density of which we want.}
  \item{n}{integer; the number of abscissa values for
    \code{\link{density}} evaluation (and plotting).}
  \item{log.bw}{logical; if true (default), the gui scrollbar is on a
    \emph{log} bandwidth scale, otherwise, simple interval.}
  \item{showvalue}{logical; if true, the value of the current (log)
    bandwidth is shown on top of the scrollbar.}
  \item{xlim}{initial \code{xlim} for plotting, see \code{\link{plot.default}}.}
  \item{do.rug}{logical indicating if \code{\link{rug}(y)} should be
    added to each plot.  This is too slow for really large sample sizes.}
  \item{from.f, to.f}{numeric giving the left and right limit of the
    bandwidth scrollbar.}
  \item{col}{color to be used for the density curve.}
}
\details{
  \code{library(tcltk)} must be working, i.e., Tcl/Tk must have been
  installed on your platform, and must have been visible during \R's
  configuration and/or installation.

  You can not only choose the bandwidth (the most important parameter),
  but also the kernel, and you can zoom in and out (in x-range only).
}
\value{
  none.\cr
  (How could this be done?  \code{tcltk} widgets run as separate processes!)
}
\author{Martin Maechler, building on \code{demo(tkdensity)}.}

%\seealso{ ~~objects to SEE ALSO as \code{\link{~~fun~~}}, ~~~ }

\examples{
data(faithful)
if(try(require(tcltk)))# still not there on broken installations
  tkdensity(faithful $ eruptions)
}
\keyword{hplot}
\keyword{dynamic}

\eof
\name{u.Datumvonheute}
\alias{u.Datumvonheute}
\alias{C.Monatsname}
\alias{C.Wochentag}
\alias{C.Wochentagkurz}
\alias{C.weekday}
\title{Datum und Uhrzeit (auf deutsch)}
\description{
  Return current date and time as a string, possibly including day of
  the week in \emph{German}.
}
\usage{
u.Datumvonheute(W.tag=2, Zeit=FALSE)

C.Monatsname
C.Wochentag
C.Wochentagkurz
C.weekday
}
\arguments{
  \item{W.tag}{logical or integer specifying you want weekday (`Wochentag').
    \code{0} or \code{FALSE} gives no,  \code{1} or \code{TRUE} gives a
    short and \code{2} the long version of the day of the week.}
  \item{Zeit}{logical or integer specifying if time ("Zeit") is desired.
        \code{0} or \code{FALSE} gives no,  \code{1} or \code{TRUE} gives a
    hours only and \code{2} hours and minutes.}
}
\value{
  A string with the current date/time, in the form specified by the arguments.

  The \code{C.*} are \code{\link{character}} vector ``constants'',
  the German ones actually used by \code{u.Datumvonheute}.
}
\author{Caterina Savi, Martin Maechler}
\seealso{\code{\link{u.date}} for a similar English version, and
  \code{\link{p.datum}} which plots.
  For English month names, etc \code{\link{month.name}}.
}
\examples{
u.Datumvonheute()
u.Datumvonheute(W.tag=1, Zeit=TRUE)
u.Datumvonheute(W.tag= FALSE, Zeit=2)
}
\keyword{utilities}

\eof
\name{u.assign0}
\alias{u.assign0}
\alias{u.get0}
\title{`Portable' assign / get functions (R / S-plus) for `frame 0'}
\description{
  \R doesn't have S' concept of \code{frame = 0}, aka \sQuote{session
    frame}.  These two function were an attempt to provide a portable
  way for working with frame 0, particularly when porting code
  \emph{from} S.  Its author, MM, no longer recommends them.
}
\usage{
u.assign0(x, value, immediate = FALSE)
u.get0(x)
}
\arguments{
  \item{x}{character string giving the \emph{name} of the object.}
  \item{value}{any \R object which is to be assigned.}
  \item{immediate}{logical, for S compatibility.  No use in \R.}
}
\author{Martin Maechler}
\note{Really don't use these anymore...}
\seealso{\code{\link{get}}, \code{\link{assign}}.}
%
\keyword{utilities}
\keyword{environment}% System, not R

\eof
\name{u.boxplot.x}
\alias{u.boxplot.x}
\title{Utility Returning x-Coordinates of Boxplot}
\description{
  Return the x-coordinates in an `n-way' side-by-side boxplot.
  This is an auxiliary function and exists mainly for backcompatibility
  with S-plus.
}
\usage{
u.boxplot.x(n, j = 1:n, fullrange = 100)
}
\arguments{
  \item{n}{number of boxplots.}
  \item{j}{indices of boxplots.}
  \item{fullrange}{x-coords as 'uniform' in \eqn{[0,fullrange]};
    (f.=100, corresponds to Splus 3.x (x = 1,2)).}
}
\value{
  a numeric vector of length \code{n}, with values inside \eqn{(0,M)} where
  \eqn{M = } \code{fullrange}.
}
\author{Martin Maechler}
\seealso{\code{\link{boxplot}}.}
\examples{
 u.boxplot.x(7) # == 8.93 22.62 36.3 ... 91.07
}
\keyword{dplot}
\keyword{utilities}

\eof
\name{u.date}
\alias{u.date}
\title{Return Date[-Time] String in `European' Format}
\description{
  Return one string of the form "day/month/year", plus "hour:minutes",
  optionally. 
}
\usage{
u.date(short=FALSE)
}
\arguments{\item{short}{logical; if \code{TRUE}, no time is given.}
}
\value{String with current date (and time).~Describe the value returned
}
\author{Martin Maechler, ca. 1992}
\seealso{\code{\link{u.Datumvonheute}}.}
\examples{
u.date()
u.date(short = TRUE)
}
\keyword{utilities}


\eof
\name{u.datumdecode}
\alias{u.datumdecode}
\title{Convert ``Numeric'' Dates}
\description{
  Daten der Form 8710230920 aufspalten in Jahr, Monat, Tag, Std, Min
}
\usage{
u.datumdecode(d, YMDHMnames = c("Jahr", "Monat", "Tag", "Std", "Min"))
}
\arguments{
  \item{d}{numeric dates in the form YYMMDDHHMM.}
  \item{YMDHMnames}{(column) names to be used for the result.}
}
% \details{
%   ~~ If necessary, more details than the __description__  above ~~
% }
\value{
  a numeric matrix (or vector) with 5 columns containing the year,
  month, etc.
}
\author{?? (someone at SfS ETH)}
\note{
  MM: This is a wrong concept, and also suffers from the ``millenium bug''
  (by using only 2 digits for the year).
}
\seealso{\R's \emph{proper} date-time coding: \code{\link{DateTimeClasses}};
  \code{\link{u.date}} etc.
}
\examples{
u.datumdecode(8710230920)
##  Jahr Monat   Tag   Std   Min
##    87    10    23     9    20

u.datumdecode(c(8710230900, 9710230920, 0210230920))
##      Jahr Monat Tag Std Min
## [1,]   87    10  23   9  00
## [2,]   97    10  23   9  20
## [3,]    2    10  23   9  20
}
\keyword{utilities}

\eof
\name{u.log}
\alias{u.log}
\title{(Anti)Symmetric Log High-Transform}
\description{
  Compute \eqn{log()} only for high values and keep low ones --
  antisymmetrically such that \code{u.log(x)} is (once) continuously
  differentiable, it computes
%Fails \deqn{f(x) = \left\{\begin{array}{ll}
%Fails    x	 & for |x| \leq c \\
%Fails    sign(x) c (1 + \log(|x|/c)) & for |x| > c \end{array}}{%
%Fails    f(x) = x  for |x| <= c  and  sign(x)*c*(1 + log(|x|/c)) for |x| >= c.}
  \eqn{f(x) = x} for \eqn{|x| \le c}{|x| <= c}  and
  \eqn{sign(x)  c\cdot(1 + log(|x|/c))}{sign(x)*c*(1 + log(|x|/c))}
  for \eqn{|x| \ge c}{|x| >= c}.
}
\usage{
u.log(x, c = 1)
}
\arguments{
  \item{x}{numeric vector to be transformed.}
  \item{c}{scalar, > 0}
}
\value{
  numeric vector of same length as \code{x}.}
}
%\references{ ~put references to the literature/web site here ~ }
\author{Martin Maechler, 24 Jan 1995}
%\seealso{ ~~objects to SEE ALSO as \code{\link{~~fun~~}}, ~~~ }
\examples{
curve(u.log, -3, 10); abline(h=0, v=0, col = "gray20", lty = 3)
curve(1 + log(x), .01,  add = TRUE, col= "brown") # simple log
curve(u.log(x,    2),   add = TRUE, col=2)
curve(u.log(x, c= 0.4), add = TRUE, col=4)
}
\keyword{arith}

\eof
\name{u.sys}
\alias{u.sys}
\title{`Portable' System function (R / S-plus)}
\description{
  Convenient call to the underlying operating system.  The main purpose
  has been to provide a function with identical UI both in S-PLUS and \R.
  MM thinks you shouldn't use this anymore, usually.
}
\usage{
u.sys(\dots, intern = TRUE)
}
\arguments{
  \item{\dots}{any number of strings -- which will be
    \code{\link{paste}()}d together and passed to \code{system}.}
  \item{intern}{logical -- note that the default is \emph{reversed} from
    the one in \code{\link{system}()}.}
}
\author{Martin Maechler}
\seealso{\code{\link{system}}, really!}
\examples{
u.sys # shows how simply the function is defined :
\dontrun{
  function (..., intern = TRUE)
  system(paste(..., sep = ""), intern = intern)
}
}
\keyword{utilities}
\keyword{environment}% System, not R

\eof
\name{unif}
\alias{unif}
\title{Nice Uniform Points in Interval}
\description{
  Give regularly spaced points on interval \eqn{[-c,c]} with mean 0
  (exactly) and variance about 1 (very close for \bold{even} \code{n}
  and larger \code{round.dig}).  Note that \eqn{c} depends on \code{n}.
}
\usage{
unif(n, round.dig = 1 + trunc(log10(n)))
}
\arguments{
  \item{n}{positive integer specifying the number of points desired.}
  \item{round.dig}{integer indicating to how many digits the result is
    rounded.}
}
\value{
  numeric vector of length \code{n}, symmetric around {0}, hence
  with exact mean \code{0}, and variance approximately 1.
}
\note{It relies on the fact that \eqn{Var(1,2,...,n) = n(n+1)/12}.
}
\author{Martin Maechler, ca 1990}
\seealso{\code{\link{runif}} for producing uniform \emph{random} numbers.}
\examples{
(u <- unif(8))
var(u)

(u. <- unif(8, 12))# more digits in result, hence precision for Var :
var(u.)
}
\keyword{arith}
\keyword{utilities}

\eof
\name{uniqueL}
\alias{uniqueL}
\title{A Reversable Version of unique()}
\description{
  A version of \code{\link{unique}} keeping enough information to
  reverse (or \emph{invert}) to the original data.
}
\usage{
uniqueL(x, isuniq = !duplicated(x))
}
\arguments{
  \item{x}{numeric vector, of length \code{n}, say.}
  \item{isuniq}{logical vector of the same length as \code{x}.  For the
    reversion to work this should select at least all unique values of
    \code{x}.}
}
\value{
  list of two components,
  \item{ix}{integer vector of indices}
  \item{xU}{vector of values from \code{x}}
  such that both \code{x[isuniq] === xU} and \code{xU[ix] === x}.
}
\author{Martin Maechler}
\seealso{\code{\link{unique}}, \code{\link{duplicated}}.}
\examples{
 x0 <- c(1:3,2:7,8:4)
 str(r0 <- uniqueL(x0))
 with(r0, xU[ix]) ## == x0 !
\testonly{
 for(n in 1:100) {
   x0 <- round((1+rpois(1,lam=1))*rnorm(40))
   r0 <- uniqueL(x0)
   stopifnot(sort(unique(x0)) == r0$xU,
	     with(r0, xU[ix]) == x0)
 }
}
}
\keyword{utilities}

\eof
\name{vcat}
\alias{vcat}
\alias{ccat}
\title{Paste Utilities -- Concatenate Strings}
\description{
  Con\bold{cat}enate vector elements or anything using
  \code{\link{paste}(*, collapse = .)}.
  These are simple short abbreviations I have been using in my own codes
  in many places.
}
\usage{
vcat(vec, sep = " ")
ccat(...)
}
\arguments{
  \item{vec, \dots}{any vector and other arguments to be pasted to together.}
  \item{sep}{the separator to use, see the \emph{Details} section.}
}
\details{The functions are really just defined as\cr
 vcat :=  \code{function(vec, sep = " ") paste(vec, collapse = sep)}

 ccat :=  \code{function(...)            paste(..., collapse = "", sep = "")}
}
\value{
  a character string (of length 1) with the concatenated arguments.
}
\author{Martin Maechler, early 1990's.}
\seealso{\code{\link{paste}}, \code{\link{as.character}},
    \code{\link{format}}. \code{\link{cat}()} is really for printing.
}
\examples{
ch <- "is"
ccat("This ", ch, " it: ", 100, "\%")
vv <- c(1,pi, 20.4)
vcat(vv)
vcat(vv, sep = ", ")
}
\keyword{print}
\keyword{utilities}

\eof
\name{xy.grid}
\alias{xy.grid}
\title{Produce regular grid matrix.}
\description{
  Produce the grid used by \code{\link{persp}}, \link{contour}, etc, as
  an \code{N x 2} matrix.
  This is really outdated by \code{\link{expand.grid}()} nowadays.
}
\usage{
xy.grid(x, y)
}
\arguments{
  \item{x,y}{any vectors of same mode.}
}
\value{
  a 2-column matrix of ``points'' for each combination of \code{x} and
  \code{y}, i.e. with \code{length(x) * length(y)} rows.
}
\author{Martin Maechler, 26 Oct 1994.}
\seealso{\code{\link{expand.grid}} which didn't exist when
  \code{xy.grid} was first devised.}
\examples{
plot(xy.grid(1:7, 10*(0:4)))

x <- 1:3 ;  y <- 10*(0:4)
xyg <- xy.grid(x,y)

## Compare with expand.grid() :
m2 <- as.matrix(expand.grid(y,x)[, 2:1])
dimnames(m2) <- NULL
stopifnot(identical(xyg, m2))
}
\keyword{array}

\eof
\name{xy.unique.x}
\alias{xy.unique.x}
\title{Uniqify (X,Y) Values using Weights}
\description{
  Given \emph{smoother} data \eqn{(x_i, y_i)} and maybe weights \eqn{w_i},
  with multiple \eqn{x_i}, use the unique x values, replacing the
  \eqn{y}'s by their (weighted) mean and updating the weights
  accordingly.
}
\usage{
xy.unique.x(x, y, w, fun.mean = mean)
}
\arguments{
  \item{x,y}{numeric vectors of same length.  Alternatively, \code{x}
    can be ``xy'' like structure.}
  \item{w}{numeric vector of non-negative weights -- or missing which
    corresponds to all weights equal.}
  \item{fun.mean}{the mean function to use.}
}
\value{
  Numeric matrix with three columns, named
  \code{x}, \code{y} and \code{w} with unique \code{x} values and
  corresponding \code{y} and weights \code{w}.
}
\author{Martin Maechler, 8 Mar 1993.}
\seealso{e.g., \code{\link{smooth.spline}} uses something like
  this internally.}
\examples{
## simple example:
xy.unique.x(c(1,1,1,2,4), 1:5)
#   x y w
# 1 1 2 3
# 2 2 4 1
# 3 4 5 1
}
\keyword{utilities}

\eof
